import subprocess
import logging
import csv
import os

class SequenceSearchResult:
    QUERY_FROM_FIELD = 'query_from'
    QUERY_TO_FIELD = 'query_to'
    QUERY_LENGTH_FIELD = 'query_length'
    HIT_FROM_FIELD = 'hit_from'
    HIT_TO_FIELD = 'hit_to'
    ALIGNMENT_LENGTH_FIELD = 'alignment_length'
    ALIGNMENT_BIT_SCORE = 'alignment_bit_score'
    ALIGNMENT_DIRECTION = 'alignment_direction'
    HIT_ID_FIELD = 'hit_id'
    QUERY_ID_FIELD = 'query_id'
    HMM_NAME_FIELD = 'hmm_name'
    ACCESSION_ID_FIELD = 'accession_id'
    PERCENT_ID_FIELD = 'percent_id'
    MISMATCH_FIELD = "mismatch"
    EVALUE_FIELD = "evalue"
    
    
    
    def __init__(self):
        self.fields = []
        self.results = []
        
    def each(self, field_names):
        """Iterate over the results, yielding a list for each result, where
        each element corresponds to the field given in the field_name parameters
        
        Parameters
        ----------
        field_names: list of str
            The names of the fields to be returned during iteration
            
        Returns
        -------
        None
        
        Exceptions
        ----------
        raises something when a field name is not in self.fields
        """
        field_ids = []
        for f in field_names:
            # below raises error if the field name is not found, so
            # don't need to account for that.
            field_ids.append(self.fields.index(f))
        
        for r in self.results:
            yield([r[i] for i in field_ids])
        
class DiamondSearchResult(SequenceSearchResult):
    @staticmethod
    def import_from_daa_file(daa_filename):
        '''Generate new results object from the output of diamond blastx/p'''
        
        # blast m8 format is
        # 'qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore
        res = DiamondSearchResult()
        res.fields = [
                       SequenceSearchResult.QUERY_ID_FIELD,
                       SequenceSearchResult.HIT_ID_FIELD,
                       SequenceSearchResult.PERCENT_ID_FIELD,
                       SequenceSearchResult.ALIGNMENT_LENGTH_FIELD,
                       SequenceSearchResult.MISMATCH_FIELD,
                       #skip
                       SequenceSearchResult.QUERY_FROM_FIELD,
                       SequenceSearchResult.QUERY_TO_FIELD,
                       SequenceSearchResult.HIT_FROM_FIELD,
                       SequenceSearchResult.HIT_TO_FIELD,
                       SequenceSearchResult.EVALUE_FIELD,
                       SequenceSearchResult.ALIGNMENT_BIT_SCORE,
                       # extras
                       SequenceSearchResult.ALIGNMENT_DIRECTION,
                       SequenceSearchResult.HMM_NAME_FIELD
                       ]
        
        cmd = "diamond view -a '%s'" % daa_filename
        logging.debug("Running cmd: %s" % cmd)
        process = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
        stdout, stderr = process.communicate()
        
        reader = csv.reader(stdout.decode('ascii').splitlines(),
                                delimiter='\t')
        if process.returncode != 0:
            raise Exception("Problem running diamond view with cmd: '%s'," 
                            "stderr was %s" % (cmd, stderr))

        for row in reader:
            # 'qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore
            #    0       1     2      3        4        5      6     7    8      9    10     11
            query_start = int(row[6])
            query_end = int(row[7])
            res.results.append([row[0],
                                 row[1],
                                 row[2],
                                 row[3],
                                 row[4],
                                 query_start,
                                 query_end,
                                 int(row[8]),
                                 int(row[9]),
                                 row[10],
                                 row[11],
                                 query_start < query_end,
                                 os.path.basename(daa_filename)
                                 ])
        return res

class HMMSearchResult(SequenceSearchResult):
    @staticmethod
    def import_from_nhmmer_table(hmmout_path):
        '''Generate new results object from the output of nhmmer search'''
        # nhmmer format is
        # qseqid queryname hmmfrom hmmto alifrom alito envfrom envto sqlen strand evalue bitscore bias description
        #   0        2        4      5      6      7      8      9    10     11     12      13     14     15     
        res=HMMSearchResult()
        res.fields = [
                       SequenceSearchResult.QUERY_ID_FIELD,
                       SequenceSearchResult.HMM_NAME_FIELD,
                       SequenceSearchResult.ALIGNMENT_LENGTH_FIELD,
                       SequenceSearchResult.QUERY_FROM_FIELD,
                       SequenceSearchResult.QUERY_TO_FIELD,
                       SequenceSearchResult.HIT_FROM_FIELD,
                       SequenceSearchResult.HIT_TO_FIELD,
                       SequenceSearchResult.ALIGNMENT_BIT_SCORE,
                       SequenceSearchResult.ALIGNMENT_DIRECTION,
                       ]
        
        for row in [x.rstrip().split() for x in open(hmmout_path) if not x.startswith('#')]:
            alifrom    = int(row[6])
            alito      = int(row[7])
            aln_length = (alito-alifrom if alito-alifrom>0 else alifrom-alito)
            res.results.append([row[0],
                                row[2],
                                aln_length,
                                int(row[4]),
                                int(row[5]),
                                alifrom,
                                alito,
                                row[13],
                                alito > alifrom
                                ])
        return res
    @staticmethod
    def import_from_hmmsearch_table(hmmout_path):
        '''Generate new results object from the output of hmmsearch search'''
        # hmmsearch format is
        # qseqid tlen queryname qlen evalue bitscore bias hmmfrom hmmto alifrom alito envfrom envto acc
        #    0    2       3      5     6        7     8      15    16     17     18      19    20   21
        res=HMMSearchResult()
        res.fields = [
                       SequenceSearchResult.QUERY_ID_FIELD,
                       SequenceSearchResult.HMM_NAME_FIELD,
                       SequenceSearchResult.ACCESSION_ID_FIELD,
                       SequenceSearchResult.QUERY_LENGTH_FIELD,
                       SequenceSearchResult.ALIGNMENT_LENGTH_FIELD,
                       SequenceSearchResult.QUERY_FROM_FIELD,
                       SequenceSearchResult.QUERY_TO_FIELD,
                       SequenceSearchResult.HIT_FROM_FIELD,
                       SequenceSearchResult.HIT_TO_FIELD,
                       SequenceSearchResult.ALIGNMENT_BIT_SCORE,
                       SequenceSearchResult.ALIGNMENT_DIRECTION,
                       ]
        
        for row in [x.rstrip().split() for x in open(hmmout_path) if not x.startswith('#')]:
            alifrom    = int(row[17])
            alito      = int(row[18])
            aln_length = (alito-alifrom if alito-alifrom>0 else alifrom-alito)
            if alito != alifrom: #this actually happens..
                res.results.append([row[0],
                                    row[3],
                                    row[4],
                                    row[5],
                                    aln_length,
                                    int(row[15]),
                                    int(row[16]),
                                    alifrom,
                                    alito,
                                    row[7],
                                    True
                                    ])
        return res
        
        
        
        
        
        
        
        
         


#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import os
import sys

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.unpack_sequences import UnpackRawReads

class Tests(unittest.TestCase):
    def test__guess_sequence_type(self):
        urr = UnpackRawReads(None)
        self.assertEqual('aminoacid', urr._guess_sequence_type_from_string('P'*10))
        self.assertEqual('aminoacid', urr._guess_sequence_type_from_string('P'*10+'T'*89))
        self.assertEqual('nucleotide', urr._guess_sequence_type_from_string('P'*10+'T'*90))
        self.assertEqual('nucleotide', urr._guess_sequence_type_from_string('A'*300+'E'*999)) #only look at the first 300bp

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

import re
import sets
import string
import logging

class Getaxnseq:
    # Perhaps in the future allow this to be set programmatically
    _taxonomic_level_names = string.split('kingdom phylum class order family genus species')
    
    def _taxonomy_line(self, level_index, taxon_array):
        
        if level_index == 0:
            return '%s,Root,%s,%s,%s,%s,,,,,,' % (taxon_array[level_index], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[level_index])
        elif level_index == 1:
            return '%s,%s,%s,%s,%s,%s,%s,,,,,' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1])
        elif level_index == 2:
            return '%s,%s,%s,%s,%s,%s,%s,%s,,,,' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1], taxon_array[2])
        elif level_index == 3:
            return '%s,%s,%s,%s,%s,%s,%s,%s,%s,,,' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1], taxon_array[2], taxon_array[3])
        elif level_index == 4:
            return '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,,' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1], taxon_array[2], taxon_array[3], taxon_array[4])
        elif level_index == 5:
            return '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1], taxon_array[2], taxon_array[3], taxon_array[4], taxon_array[5])
        elif level_index == 6:
            return '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s' % (taxon_array[level_index], taxon_array[level_index-1], self._taxonomic_level_names[level_index], taxon_array[level_index], 'Root', taxon_array[0], taxon_array[1], taxon_array[2], taxon_array[3], taxon_array[4], taxon_array[5], taxon_array[6])
        else:
            raise Exception("Programming error, found too many taxonomic levels!")
    
    def read_taxtastic_taxonomy_and_seqinfo(self, taxonomy_io, seqinfo_io):
        '''Read the taxonomy and seqinfo files into a dictionary of 
        sequence_name => taxonomy, where the taxonomy is an array of lineages
        given to that sequence.
        
        Possibly this method is unable to handle the full definition of these
        files? It doesn't return what each of the ranks are, for starters.
        Doesn't deal with duplicate taxon names either.
        '''
        
        # read in taxonomy file
        lineages = [] #array of hashes where each taxon points to its parent taxon's name
        taxon_to_lineage_index = {}
        expected_number_of_fields = None
        for line in taxonomy_io:
            splits = line.strip().split(',')
            if expected_number_of_fields is None:
                expected_number_of_fields = len(splits)
                lineages = [{}]* (expected_number_of_fields-4)
                continue #this is the header line
            elif len(splits) != expected_number_of_fields:
                raise Exception("Encountered error parsing taxonomy file, expected %i fields but found %i on line: %s" %
                                (expected_number_of_fields, len(splits), line))
            # e.g. 'tax_id,parent_id,rank,tax_name,root,kingdom,phylum,class,order,family,genus,species
            tax_id = splits[0]
            parent_id = splits[1]
            
            try:
                lineage_index = splits.index('')-5
            except ValueError:
                lineage_index = len(splits)-5
            taxon_to_lineage_index[tax_id] = lineage_index
            lineages[lineage_index][tax_id] = parent_id
        
        taxonomy_dictionary = {}
        for i, line in enumerate(seqinfo_io):
            if i==0: continue #skip header line
            
            splits = line.strip().split(',')
            if len(splits) != 2:
                raise Exception("Bad formatting of seqinfo file on this line: %s" % line)
            
            seq_name = splits[0]
            taxon = splits[1]
            lineage_index = taxon_to_lineage_index[taxon]
            if lineage_index==0:
                # Don't include Root in the taxonomy
                taxonomy_dictionary[seq_name] = []
            else:
                full_taxonomy_rev = []
                while lineage_index > 0:
                    full_taxonomy_rev.append(taxon)
                    taxon = lineages[lineage_index][taxon]
                    lineage_index = lineage_index-1
                taxonomy_dictionary[seq_name] = list(reversed(full_taxonomy_rev))
            
        return taxonomy_dictionary
    
    def write_taxonomy_and_seqinfo_files(self, taxonomies, output_taxonomy_file, output_seqinfo_file):
        '''Write out taxonomy and seqinfo files as required by taxtastic
        from known taxonomies
        
        Parameters
        ----------
        taxonomies:
            hash of taxon_id to array of taxonomic information
        output_taxonomy_file:
            write taxtastic-compatible 'taxonomy' file here
        output_seqinfo_file:
            write taxtastic-compatible 'seqinfo' file here'''
        
        first_pass_id_and_taxonomies = []
        meaningless_taxonomic_names = sets.Set(['k__', 'd__', 'p__', 'c__', 'o__', 'f__', 'g__', 's__'])

        for taxon_id, tax_split in taxonomies.iteritems():
            # Replace spaces with underscores e.g. 'Candidatus my_genus'
            for idx, item in enumerate(tax_split):
                tax_split[idx] = re.sub('\s+', '_', item.strip())

            # Remove 'empty' taxononomies e.g. 's__'
            tax_split = [item for item in tax_split if item not in meaningless_taxonomic_names]

            # Add this fixed up list to the list
            first_pass_id_and_taxonomies.append([taxon_id]+tax_split)

        # Find taxons that have multiple parents, building a hash of parents as we go (i.e. a tree of taxonomies embedded in a hash)
        #
        # Assumes that no two taxonomic labels are the same when they are from different
        # taxonomic levels. When there are children with multiple parents at the
        # same taxonomic label then these are warned about and worked around.
        parents = {} #hash of taxon to immediate parent
        known_duplicates = sets.Set([])
        for j, array in enumerate(first_pass_id_and_taxonomies):
            taxonomy = array[1:]
            for i, tax in enumerate(taxonomy):
                if i==0: continue #top levels don't have parents
                ancestry = taxonomy[i-1]
                if parents.has_key(tax):
                    if parents[tax] != ancestry:

                        dup = "%s%s" %(parents[tax], ancestry)
                        # don't report the same problem several times
                        if dup not in known_duplicates:
                            print " %s '%s' with multiple parents %s and %s" % (array[0], tax, parents[tax], ancestry)
                            known_duplicates.add(dup)

                        # fix the current one
                        new_name_id = 1
                        new_name = "%se%s" % (tax, new_name_id)

                        while parents.has_key(new_name) and parents[new_name] != ancestry:
                            new_name_id += 1
                            new_name = "%se%s" % (tax, new_name_id)

                        first_pass_id_and_taxonomies[j][i+1] = new_name
                        taxonomy[i] = new_name
                        parents[new_name] = ancestry
                else:
                    # normal case, seeing a new taxon and parent for the first time

                    parents[tax] = ancestry

        # Write the sequence file
        with open(output_seqinfo_file, 'w') as seqout:
            # write header
            seqout.write('seqname,tax_id\n')
            # write each taxonomic association
            for array in first_pass_id_and_taxonomies:
                sequence_id = array[0]
                if len(array)==1:
                    most_specific_taxonomic_affiliation = 'Root'
                elif len(array)-1 > len(self._taxonomic_level_names):
                    logging.warn("Excluding the more specific parts of the taxonomic assignment '%s' assigned to sequence '%s' due to it having more levels than this code can currently accommodate" %\
                                 (array[1:], sequence_id))
                    most_specific_taxonomic_affiliation = array[len(self._taxonomic_level_names)]
                else:  
                    most_specific_taxonomic_affiliation = array[-1]
                seqout.write("%s,%s\n" % (array[0], 
                                          most_specific_taxonomic_affiliation))

        # Write the taxonomy file
        noted_taxonomies = sets.Set([])
        with open(output_taxonomy_file, 'w') as seqout:
            # write header and root line
            seqout.write('tax_id,parent_id,rank,tax_name,root,kingdom,phylum,class,order,family,genus,species\n')
            seqout.write('Root,Root,root,Root,Root,,,,,,,\n')
            # write all the taxonomies
            for array in first_pass_id_and_taxonomies:
                taxons = array[1:]
                for i, tax in enumerate(taxons):
                    if i < len(self._taxonomic_level_names):
                        line = self._taxonomy_line(i, taxons[:(i+1)])
                        if line not in noted_taxonomies:
                            seqout.write(line+"\n")
                            noted_taxonomies.add(line)






from graftm.sequence_search_results import DiamondSearchResult
import tempfile
import extern
import os
from graftm.unpack_sequences import UnpackRawReads

class Diamond:
    def __init__(self, database, threads=None, evalue=None):
        self._database = database
        self._threads = threads
        self._evalue = evalue
        
    def run(self, input_sequence_file, input_sequence_type, daa_file_basename=None):
        '''Run input sequences in either blastp or blastx mode against the
        database specified in __init__.
            
        Parameters
        ----------
        input_sequence_file: str
            path to query sequences
        input_sequence_type: either 'nucleotide' or 'protein'
            the input_sequences are this kind of sequence
            
        Returns
        -------
        DiamondSearchResult
        '''
        
        cmd_list = ["diamond"]
        if input_sequence_type == UnpackRawReads.PROTEIN_SEQUENCE_TYPE:
            cmd_list.append('blastp')
        elif input_sequence_type == UnpackRawReads.NUCLEOTIDE_SEQUENCE_TYPE:
            cmd_list.append('blastx')
        else:
            raise Exception("Programming error")
        
        basename = daa_file_basename
        if basename is None:
            with tempfile.NamedTemporaryFile(prefix='graftm_diamond') as t:
                # we are just stealing the name, don't need the file itself
                basename = t.name
            
        for c in ['-k 1',
                  "-d",
                    self._database,
                    "-q",
                    "%s" % input_sequence_file,
                    "-a",
                    basename]:
            cmd_list.append(c)
        if self._threads:
            cmd_list.append("--threads")
            cmd_list.append(str(self._threads))
        if self._evalue:
            cmd_list.append("--evalue")
            cmd_list.append(str(self._evalue))
        
        cmd = ' '.join(cmd_list)
        extern.run(cmd)
        
        daa_name = "%s.daa" % basename
        res = DiamondSearchResult.import_from_daa_file(daa_name)
        
        if daa_file_basename is None:
            # Diamond makes an extra file, need to remove this
            os.remove(daa_name)
            
        return res
            
            
            
                


#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import os
import sys
from string import split

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.deduplicator import Deduplicator
from graftm.sequence_io import Sequence

class Tests(unittest.TestCase):
    def s(self, seqs):
        to_return = []
        current_seq = None
        for bit in split(seqs,' '):
            if current_seq:
                current_seq.seq = bit
                to_return.append(current_seq)
                current_seq = None
            else:
                current_seq = Sequence(bit, '')
        return to_return
    
    def nd(self, seqs):
        return sorted([' '.join([x.name for x in s])
                       for s in self.deduplicator.deduplicate(seqs)])
        
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.deduplicator = Deduplicator()
        self.d = self.deduplicator
        
    def test_deduplicate_hello_world(self):
        seqs = self.s('1 AAA 2 AAT')
        self.assertEqual(['1','2'], self.nd(seqs))
                
    def test_deduplicate_one_dup(self):
        self.assertEqual(['1 2'], self.nd(self.s('1 AAA 2 AAA')))
                
    def test_deduplicate_one_dup_and_one_not(self):
        self.assertEqual(['1 3','2'], self.nd(self.s('1 AAA 2 ATA 3 AAA')))
        
    def test_lca(self):
        self.assertEqual(['1 3','2'], self.nd(self.s('1 AAA 2 ATA 3 AAA')))
        dees = self.d.deduplicate(self.s('1 AAA 2 ATA 3 AAA'))
        self.assertEqual([['one','two'], ['one','two','five']],
                         self.d.lca_taxonomy(dees, 
                                             {'1': ['one','two','four'],
                                              '2': ['one','two','five'],
                                              '3': ['one','two','three']}))
        
    def test_lca_more_specific_then_less(self):
        self.assertEqual(['1 3','2'], self.nd(self.s('1 AAA 2 ATA 3 AAA')))
        dees = self.d.deduplicate(self.s('1 AAA 2 ATA 3 AAA'))
        self.assertEqual([['one'], ['one','two']],
                         self.d.lca_taxonomy(dees, 
                                             {'1': ['one','two','four'],
                                              '2': ['one','two'],
                                              '3': ['one']}))
        
    def test_group_of_three_deduplication(self):
        dees = self.d.deduplicate(self.s('1 AAA 2 AAA 3 AAA'))
        self.assertEqual([['one','two','three']],
                         self.d.lca_taxonomy(dees, 
                                             {'1': ['one','two','three','1'],
                                              '2': ['one','two','three','2'],
                                              '3': ['one','two','three','3']}))
        
if __name__ == "__main__":
    unittest.main()



import logging
import tempfile
import shutil
import extern

from graftm.hmmer import Hmmer
from graftm.unpack_sequences import UnpackRawReads
from graftm.sequence_io import SequenceIO

class Bootstrapper:
    
    # Define constants.
    DIAMOND_SEARCH_METHOD = "diamond"
    HMM_SEARCH_METHOD = "hmmsearch"    

    def __init__(self, **kwargs):
        '''
        Parameters
        ----------
        search_hmm_files: list of str
            list of HMM files to search with
        graftm_package: GraftMPackage
            use the search HMMs from this graftm package in addition to
            those specified in search_hmm_files
        threads, evalue, min_orf_length, restrict_read_length:
            as per hmmer.search_and_extract_orfs_matching_protein_database
        '''
        self.search_hmm_files = kwargs.pop('search_hmm_files',[])
        self.maximum_range = kwargs.pop('maximum_range',None)
        self.threads = kwargs.pop('threads',None)
        self.evalue = kwargs.pop('evalue',None)
        self.min_orf_length = kwargs.pop('min_orf_length',None)
        graftm_package = kwargs.pop('graftm_package',None)
        if len(kwargs) > 0:
            raise Exception("Unexpected arguments detected: %s" % kwargs)
        
        if graftm_package:
            self.diamond_database = graftm_package.diamond_database_path()
            self.unaligned_sequence_database = graftm_package.unaligned_sequence_database_path()
            if self.search_hmm_files is None:
                self.search_hmm_files = []
            for h in graftm_package.search_hmm_paths():
                if h not in self.search_hmm_files:
                    self.search_hmm_files.append(h)
            if self.maximum_range is None:
                self.maximum_range = graftm_package.maximum_range()
        else:
            self.diamond_database = None
            self.unaligned_sequence_database = None
            
        if len(kwargs) > 0:
            raise Exception("Unexpected arguments detected: %s" % kwargs)
                    
        
    def generate_bootstrap_database_from_contigs(self, contig_files, output_database_file, 
                                                 search_method):
        '''Given a collection of search_hmm_files, search the contigs in 
        contig_files, and generate an HMM from the resulting hits, outputting
        it as output_database_file.
        
        Parameters
        ----------
        contig_files: list of str
            list of files to search
        output_database_file: str
            path to output file
        search_method: str
            "diamond" or "hmmsearch", to specify search method to use and what
            type of database to build. 
        
        Returns
        -------
        True if genes were recovered, else False'''
        
        hmmer = Hmmer(self.search_hmm_files)
        seqio = SequenceIO()
        if search_method == self.DIAMOND_SEARCH_METHOD:
            if self.diamond_database == None or self.unaligned_sequence_database == None:
                logging.warning("Cannot bootstrap continue with no diamond database or unaligned sequences.") 
                return False
        
        with tempfile.NamedTemporaryFile(prefix='graftm_bootstrap_orfs') as orfs:
            logging.info("Finding bootstrap hits in provided contigs..")
            for contig_file in contig_files:
                logging.debug("Finding bootstrap hits in %s.." % contig_file)
                unpack = UnpackRawReads(contig_file)
                
                with tempfile.NamedTemporaryFile(prefix='graftm_bootstrap') as \
                                                        hit_reads_orfs_fasta:
                    # search and extract matching ORFs
                    with tempfile.NamedTemporaryFile(prefix='graftm_bootstrap2') as \
                                                        hmmsearch_output_table:
                        with tempfile.NamedTemporaryFile(prefix='graftm_bootstrap3') as \
                                                        hit_reads_fasta:
                            hmmer.search_and_extract_orfs_matching_protein_database(\
                                    unpack,
                                    search_method,
                                    self.maximum_range,
                                    self.threads,
                                    self.evalue,
                                    self.min_orf_length,
                                    None,
                                    (self.diamond_database if self.diamond_database else None),
                                    hmmsearch_output_table.name,
                                    hit_reads_fasta.name,
                                    hit_reads_orfs_fasta.name)
                    # Append to the file
                    shutil.copyfileobj(open(hit_reads_orfs_fasta.name), orfs)
            
            # Now have a fasta file of ORFs.
            # Check to make sure the file is not zero-length
            orfs.flush()
            

            
            with tempfile.NamedTemporaryFile(prefix="graftm_bootstrap_aln") as aln:
                    
                if search_method == self.HMM_SEARCH_METHOD:
                    
                    # Check that there is more than one sequence to align.
                    if len(seqio.read_fasta_file(orfs.name)) <= 1:# Just to build on this, you need to check if there is > 1 hit
                                                                  # otherwise mafft will fail to align, causing a crash when hmmbuild is 
                                                                  # run on an empty file.
                        logging.warn("Failed to find two or more matching ORFs in the bootstrap contigs")
                        return False
                    
                    # Run mafft to align them
                    cmd = "mafft --auto %s >%s" % (orfs.name, aln.name)
                    logging.info("Aligning bootstrap hits..")
                    extern.run(cmd)
                
                    # Run hmmbuild to create an HMM
                    cmd = "hmmbuild --amino %s %s >/dev/null" % (output_database_file, aln.name)
                    logging.info("Building HMM from bootstrap hits..")

                    extern.run(cmd)
                
                elif search_method == self.DIAMOND_SEARCH_METHOD:

                    # Concatenate database with existing database
                    with tempfile.NamedTemporaryFile(prefix="concatenated_database") as databasefile:
                        for f in [orfs.name, self.unaligned_sequence_database]:
                            for line in open(f):
                                databasefile.write(line)
                        databasefile.flush()
                                
                        # Run diamond make to create a diamond database
                        cmd = "diamond makedb --in '%s' -d '%s'" % (databasefile.name, output_database_file)
                        logging.info("Building a diamond database from bootstrap hits..")
                        extern.run(cmd)
                
                else:
                    raise Exception("Search method not recognised: %s" % search_method)
                    return False
                
                return True
            
                





class Sequence:
    def __init__(self, name, seq):
        self.name = name
        self.seq = seq

class SequenceIO:
    # Stolen from https://github.com/lh3/readfq/blob/master/readfq.py
    def _readfq(self, fp): # this is a generator function
        last = None # this is a buffer keeping the last unprocessed line
        while True: # mimic closure; is it a bad idea?
            if not last: # the first record or a record following a fastq
                for l in fp: # search for the start of the next record
                    if l[0] in '>@': # fasta/q header line
                        last = l[:-1] # save this line
                        break
            if not last: break
            name, seqs, last = last[1:].partition(" ")[0], [], None
            for l in fp: # read the sequence
                if l[0] in '@+>':
                    last = l[:-1]
                    break
                seqs.append(l[:-1])
            if not last or last[0] != '+': # this is a fasta record
                yield name, ''.join(seqs), None # yield a fasta record
                if not last: break
            else: # this is a fastq record
                seq, leng, seqs = ''.join(seqs), 0, []
                for l in fp: # read the quality
                    seqs.append(l[:-1])
                    leng += len(l) - 1
                    if leng >= len(seq): # have read enough quality
                        last = None
                        yield name, seq, ''.join(seqs); # yield a fastq record
                        break
                if last: # reach EOF before reading enough quality
                    yield name, seq, None # yield a fasta record instead
                    break
                
    def read_fasta_file(self, path_to_fasta_file):
        seqs = []
        for name, seq, _ in self._readfq(open(path_to_fasta_file)):
            seqs.append(Sequence(name, seq))
        return seqs
    
    def write_fasta_file(self, sequence_objects, path_to_fasta_file):
        with open(path_to_fasta_file,'w') as f:
            for s in sequence_objects:
                f.write(">")
                f.write(s.name)
                f.write("\n")
                f.write(s.seq)
                f.write("\n")
            



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import tempfile
import os
import sys

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.sequence_search_results import HMMSearchResult, SequenceSearchResult

class Tests(unittest.TestCase):
    def test_whacky_directions(self):
        hmmout = '''#                                                                                                       --- full sequence --- -------------- this domain -------------   hmm coord   ali coord   env coord
2524288035           -            897 DNGNGWU00005_mingle_output_good_seqs_Tx7Fta.aln -            808   1.9e-16   49.0   0.7   1   2   2.5e-15   2.5e-15   45.4   0.0   283   406     8   148     2   156 0.78 G480DRAFT_04506 small GTP-binding protein domain [Blautia producta DSM 2950]
2524288035           -            897 DNGNGWU00005_mingle_output_good_seqs_Tx7Fta.aln -            808   1.9e-16   49.0   0.7   2   2      0.91      0.91   -2.8   0.1   142   142   706   706   601   815 0.58 G480DRAFT_04506 small GTP-binding protein domain [Blautia producta DSM 2950]
2524285235           -            530 DNGNGWU00005_mingle_output_good_seqs_Tx7Fta.aln -            808   2.1e-13   38.9   0.0   1   1   3.9e-13   3.9e-13   38.1   0.0   326   397    78   149    61   164 0.90 G480DRAFT_01694 bacterial peptide chain release factor 3 (bRF-3) [Blautia producta DSM 2950]
#
# Program:         hmmsearch
# Version:         3.1b2 (February 2015)
# Pipeline mode:   SEARCH
# Query file:      test/data/DNGNGWU00005_mingle_output_good_seqs.hmm
# Target file:     -
# Option settings: hmmsearch -o /dev/null --domtblout /tmp/single.graftm/single/single.hmmout.csv --noali -E 1e-5 --cpu 5 test/data/DNGNGWU00005_mingle_output_good_seqs.hmm - 
# Current dir:     /home/ben/git/graftM
# Date:            Sat Aug 22 22:23:06 2015
# [ok]
'''
        with tempfile.NamedTemporaryFile(prefix='graftm_test_sequence_search_result') as f:
            f.write(hmmout)
            f.flush()
            
            res = HMMSearchResult.import_from_hmmsearch_table(f.name)
            lres = list(res.each([SequenceSearchResult.QUERY_ID_FIELD,
                                    SequenceSearchResult.ALIGNMENT_DIRECTION]))
            self.assertEqual([['2524288035',True],['2524285235',True]], lres)
        
        
        
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import tempfile
import os
import sys

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.hmmer import Hmmer

class Tests(unittest.TestCase):
    def test_merg_aln(self):
        forward_reads='''>no_overlap
-------CGTATGCAACCTACCTT---------------------------------------
>overlap_all_match
-------CGTATGCAACCTACCTT---------------------------------------
>overlap_mismatch_in_reverse
-------CGTATGCAACCTACCTT---------------------------------------
>overlap_mismatch_in_forward
-------CGTATGCAACCTTCCTT---------------------------------------
>complete_overlap_all_match
-------CGTATGCAACCTACCTT---------------------------------------
>complete_overlap_mismatch
-------CGTATGCATCCTACCTT---------------------------------------'''
        reverse_reads='''>no_overlap
---------------------------------------CGTATGCAACCTACCTT-------
>overlap_all_match
----------------CCTACCTTCAACCTACCTT----------------------------
>overlap_mismatch_in_reverse
----------------CCTTCCTTCAACCTACCTT----------------------------
>overlap_mismatch_in_forward
----------------CCTACCTTCAACCTACCTT----------------------------
>complete_overlap_all_match
-------CGTATGCAACCTACCTT---------------------------------------
>complete_overlap_mismatch
-------CGTTTGCAAGCTACCTT---------------------------------------'''
        expected_aln='''>no_overlap
-------CGTATGCAACCTACCTT---------------CGTATGCAACCTACCTT-------
>overlap_all_match
-------CGTATGCAACCTACCTTCAACCTACCTT----------------------------
>overlap_mismatch_in_reverse
-------CGTATGCAACCTACCTTCAACCTACCTT----------------------------
>overlap_mismatch_in_forward
-------CGTATGCAACCTTCCTTCAACCTACCTT----------------------------
>complete_overlap_all_match
-------CGTATGCAACCTACCTT---------------------------------------
>complete_overlap_mismatch
-------CGTATGCATCCTACCTT---------------------------------------'''.split()
        with tempfile.NamedTemporaryFile(suffix='_forward.fa') as forward_file:
            with tempfile.NamedTemporaryFile(suffix='_reverse.fa') as reverse_file:
                with tempfile.NamedTemporaryFile(suffix='.fa') as output_file:
                    forward_file.write(forward_reads)
                    reverse_file.write(reverse_reads)
                    forward_file.flush()
                    reverse_file.flush()
                    Hmmer(None).merge_forev_aln([forward_file.name],[reverse_file.name],[output_file.name])
                    count = 0
                    for line in open(output_file.name):
                        self.assertEqual(expected_aln[count], line.strip())
                        count += 1
                    self.assertEqual(count, len(open(output_file.name).readlines()))

if __name__ == "__main__":
    unittest.main()



import os

class GraftMFiles:
    
    def __init__(self, old_title, outdir, direction):
        if direction == 'forward' or direction == 'reverse':
            self.basename = os.path.join(direction, old_title + '_' + direction)
        elif direction == False:
            self.basename = old_title
        else:
            raise Exception('Programming Error.')   
         
        self.outdir = outdir
    
    def search_otu_table(self):
        return os.path.join(self.outdir, "search_otu_table.txt")
    
    def hmmsearch_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s.hmmout.csv" % self.basename)
    
    def diamond_search_output_basename(self, out_path):
        '''Does not include the .daa part that diamond creates'''
        return os.path.join(self.outdir, out_path, "%s_diamond_search" % self.basename)
    
    def diamond_assignment_output_basename(self, out_path):
        '''Does not include the .daa part that diamond creates'''
        return os.path.join(self.outdir, out_path, "%s_diamond_assignment" % out_path)
    
    def read_tax_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_read_tax.tsv" % out_path)

    def jplace_output_path(self):
        return "placements.jplace"
    
    def euk_free_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_euk_free.fa" % self.basename)
    
    def euk_contam_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_euk_contam.txt" % self.basename)
    
    def summary_table_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_count_table.txt" % self.basename)
    
    def krona_output_path(self):
        return os.path.join(self.outdir, "krona.html")
    
    def aligned_fasta_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_hits.aln.fa" % self.basename)

    def orf_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_orf" % self.basename)

    def orf_titles_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_orf.titles" % self.basename)

    def orf_fasta_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_orf.fa" % self.basename)

    def conv_output_for_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_conv_for.faa" % self.basename)
    
    def output_for_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_for.faa" % self.basename)
    
    def output_rev_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_rev.faa" % self.basename)
    
    def conv_output_rev_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_conv_rev.faa" % self.basename)
    
    def comb_aln_fa(self):
        return os.path.join(self.outdir, "combined_alignment.aln.fa")

    def fa_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_hits.fa" % self.basename)     
        
    def readnames_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_readnames.txt" % self.basename)
    
    def sto_output_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s.sto" % self.basename)

    def basic_stats_path(self):
        return os.path.join(self.outdir, "basic_stats.txt")

    def for_aln_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_for_aln.fa" % self.basename)
        
    def rev_aln_path(self, out_path):
        return os.path.join(self.outdir, out_path, "%s_rev_aln.fa" % self.basename)
    
    def combined_biom_output_path(self):
        return os.path.join(self.outdir, "graftm.biom")
    
    def combined_summary_table_output_path(self):
        return os.path.join(self.outdir, "combined_count_table.txt")
    
    def bootstrap_hmm_path(self):
        return os.path.join(self.outdir, "bootstrap.hmm")
      
    def base(self, out_path):
        return os.path.join(self.outdir, out_path, "%s" % self.basename)


#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

from graftm.graftm_package import GraftMPackage


import unittest
import os.path
import sys
import tempfile
import subprocess
import logging
import extern

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.bootstrapper import Bootstrapper

path_to_script = os.path.join(os.path.dirname(os.path.realpath(__file__)),'..','bin','graftM')
path_to_data = os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')

class Tests(unittest.TestCase):
    def test_hello_world(self):
        boots = Bootstrapper(search_hmm_files = [os.path.join(path_to_data,'bootstrapper','DNGNGWU00001.hmm')],
                             evalue='1e-5',
                             maximum_range=1000,
                             threads=1)
        with tempfile.NamedTemporaryFile() as tf:
            self.assertEqual(True,
                             boots.generate_bootstrap_database_from_contigs(\
                                [os.path.join(path_to_data,'bootstrapper','contigs.fna')],
                                tf.name,
                                "hmmsearch"))

            self.assertEqual("HMMER3/f [3.1b2 | February 2015]\n",
                             subprocess.check_output("head -n1 %s" % tf.name,
                                                     shell=True))
            self.assertEqual('NSEQ  2\n', open(tf.name).readlines()[10])

    def test_hello_world_diamond(self):
        gpkg=os.path.join(path_to_data, "bootstrapper", "D1_gpkg_for_diamond.gpkg")
        boots = Bootstrapper(search_hmm_files = [os.path.join(path_to_data,'bootstrapper','DNGNGWU00001.hmm')],
                             evalue='1e-5',
                             maximum_range=1000,
                             threads=1,
                             graftm_package=GraftMPackage.acquire(gpkg))
        with tempfile.NamedTemporaryFile() as tf:
            self.assertEqual(True,
                             boots.generate_bootstrap_database_from_contigs(\
                                [os.path.join(path_to_data,'bootstrapper','diamond_bootstrap_contigs.fna')],
                                tf.name,
                                "diamond"))

    def test_no_hits(self):
        boots = Bootstrapper(search_hmm_files = [os.path.join(path_to_data,'bootstrapper','DNGNGWU00001.hmm')],
                             evalue='1e-5',
                             maximum_range=1000,
                             threads=1)
        with tempfile.NamedTemporaryFile(suffix=".fasta") as contigs:
            contigs.write(">contig\n")
            contigs.write('A'*300+"\n")
            contigs.flush()
            with tempfile.NamedTemporaryFile() as tf:
                self.assertEqual(False,
                             boots.generate_bootstrap_database_from_contigs(\
                                [contigs.name],
                                tf.name,
                                "hmmsearch"))
                
    def test_bootstrap_executable(self):
        with tempfile.NamedTemporaryFile() as tf:
            cmd = '%s bootstrap --verbosity 5 --contigs %s --output_hmm %s --search_hmm_files %s' % (path_to_script,
                                                                                              os.path.join(path_to_data,'bootstrapper','contigs.fna'),
                                                                                              tf.name,
                                                                                              os.path.join(path_to_data,'bootstrapper','DNGNGWU00001.hmm'))
            extern.run(cmd)
            self.assertEqual("HMMER3/f [3.1b2 | February 2015]\n",
                             subprocess.check_output("head -n1 %s" % tf.name,
                                                     shell=True))
            self.assertEqual('NSEQ  2\n', open(tf.name).readlines()[10])


if __name__ == "__main__":
    logging.basicConfig(level=logging.ERROR)
    unittest.main()




from graftm.deduplicator import Deduplicator
from graftm.sequence_io import SequenceIO
from graftm.orfm import OrfM
import logging
import os

class Clusterer:

    def __init__(self):
        self.clust = Deduplicator()
        self.seqio = SequenceIO()
        self.seq_library = {}

        self.orfm_regex = OrfM.regular_expression()


    def uncluster_annotations(self, input_annotations, reverse_pipe):
        '''
        Update the annotations hash provided by pplacer to include all
        representatives within each cluster

        Parameters
        ----------
        input_annotations : hash
            Classifications for each representative sequence of the clusters.
            each key being the sequence name, and the entry being the taxonomy
            string as a list.
        reverse_pipe : bool
            True/False, whether the reverse reads pipeline is being followed.

        Returns
        -------
        output_annotations : hash
            An updated version of the above, which includes all reads from
            each cluster
        '''
        output_annotations = {}
        for placed_alignment_file_path, clusters in self.seq_library.iteritems():

            if reverse_pipe and placed_alignment_file_path.endswith("_reverse_clustered.fa"): continue
            placed_alignment_file = os.path.basename(placed_alignment_file_path)
            cluster_classifications = input_annotations[placed_alignment_file]

            if reverse_pipe:
                placed_alignment_base = placed_alignment_file.replace('_forward_clustered.fa', '')
            else:
                placed_alignment_base = placed_alignment_file.replace('_clustered.fa', '')
            output_annotations[placed_alignment_base] = {}
            for rep_read_name, rep_read_taxonomy in cluster_classifications.iteritems():

                if reverse_pipe:
                    orfm_regex = OrfM.regular_expression()
                    clusters={(orfm_regex.match(key).groups(0)[0] if orfm_regex.match(key) else key): item for key, item in clusters.iteritems()}
                for read in clusters[rep_read_name]:
                    output_annotations[placed_alignment_base][read.name] = rep_read_taxonomy

        return output_annotations

    def cluster(self, input_fasta_list, reverse_pipe):
        '''
        cluster - Clusters reads at 100% identity level and  writes them to
        file. Resets the input_fasta variable as the FASTA file containing the
        clusters.

        Parameters
        ----------
        input_fasta_list : list
            list of strings, each a path to input fasta files to be clustered.
        reverse_pipe : bool
            True/False, whether the reverse reads pipeline is being followed.
        Returns
        -------
        output_fasta_list : list
            list of strings, each a path to the output fasta file to which
            clusters were written to.
        '''
        output_fasta_list = []
        for input_fasta in input_fasta_list:
            output_path  = input_fasta.replace('_hits.aln.fa', '_clustered.fa')
            cluster_dict = {}

            logging.debug('Clustering reads')
            reads=self.seqio.read_fasta_file(input_fasta) # Read in FASTA records
            logging.debug('Found %i reads' % len(reads)) # Report number found
            clusters=self.clust.deduplicate(reads) # Cluster redundant sequences
            logging.debug('Clustered to %s groups' % len(clusters)) # Report number of clusters
            logging.debug('Writing representative sequences of each cluster to: %s' % output_path) # Report the name of the file

            self.seqio.write_fasta_file(
                                        [x[0] for x in clusters],
                                        output_path
                                        ) # Choose the first sequence to write to file as representative (all the same anyway)
            for cluster in clusters:
                cluster_dict[cluster[0].name]=cluster # assign the cluster to the dictionary
            self.seq_library[output_path]= cluster_dict

            output_fasta_list.append(output_path)

        return output_fasta_list






import extern
import os
import itertools
import logging
import tempfile
import subprocess

from Bio import SeqIO
from collections import OrderedDict

from graftm.timeit import Timer
from graftm.hmmsearcher import HmmSearcher, NhmmerSearcher
from graftm.orfm import OrfM, ZcatOrfM
from graftm.diamond import Diamond
from graftm.sequence_search_results import SequenceSearchResult, HMMSearchResult
from graftm.readHmmTable import HMMreader
from graftm.db_search_results import DBSearchResult

FORMAT_FASTA = "FORMAT_FASTA"
FORMAT_FASTQ = "FORMAT_FASTQ"
FORMAT_FASTQ_GZ = "FORMAT_FASTQ_GZ"
FORMAT_FASTA_GZ = "FORMAT_FASTA_GZ"
PIPELINE_AA = "P"
PIPELINE_NT = "D"
PREVIOUS_SPAN_CUTOFF = 0.25
T = Timer()

class InterleavedFileError(Exception):
    pass

class Hmmer:

    def __init__(self, search_hmm, aln_hmm=None):
        self.search_hmm = search_hmm
        self.aln_hmm = aln_hmm

    def _get_sequence_directions(self, search_result):
        sequence_directions = {}
        for result in search_result:
            for hit in result.each([SequenceSearchResult.QUERY_ID_FIELD, SequenceSearchResult.ALIGNMENT_DIRECTION]):
                sequence_directions[hit[0]]  ={"strand":[hit[1]], "entry": []}
        return sequence_directions


    def _hmmalign(self, input_path, directions, pipeline):
        '''
        Align reads to the aln_hmm. Receives unaligned sequences and
        aligns them.

        Parameters
        ----------
        input_path : str
            Filename of unaligned hits to be aligned
        directions : dict
            dictionary containing read names as keys, and complement
            as the entry (True=Forward, False=Reverse)
        Returns
        -------
        list of two strings, where each string is a path to an aligned FASTA
        file (forward and reverse files respectively)
        '''

        for_file      = tempfile.NamedTemporaryFile(prefix='for_file', suffix='.fa').name
        rev_file      = tempfile.NamedTemporaryFile(prefix='rev_file', suffix='.fa').name
        for_conv_file = tempfile.NamedTemporaryFile(prefix='for_conv_file', suffix='.fa').name
        rev_conv_file = tempfile.NamedTemporaryFile(prefix='rev_conv_file', suffix='.fa').name
        if pipeline == PIPELINE_AA:
            reverse_direction_reads_present=False
        else:
            reverse_direction_reads_present=False in directions.values()

        # Align input reads to a specified hmm.
        if reverse_direction_reads_present:  # Any that are in the reverse direction would be True
            reverse = []
            forward = []
            records = list(SeqIO.parse(open(input_path), 'fasta'))

            # Split the reads into reverse and forward lists
            for record in records:
                read_id = record.id

                if directions[read_id] == True:
                    forward.append(record)
                elif directions[read_id] == False:
                    reverse.append(record)
                else:
                    raise Exception(logging.error('Programming error: hmmalign'))
                    exit(1)

            logging.debug("Found %i forward direction reads" % len(forward))
            logging.debug("Found %i reverse direction reads" % len(reverse))

            # Write reverse complement and forward reads to files
            with open(for_file, 'w') as for_aln:
                logging.debug("Writing forward direction reads to %s" % for_file)
                for record in forward:
                    for_aln.write('>' + record.id + '\n')
                    for_aln.write(str(record.seq) + '\n')
                    # HMMalign and convert to fasta format
            if any(forward):
                self.hmmalign_sequences(self.aln_hmm, for_file, for_conv_file)
            else:
                cmd = 'touch %s' % (for_conv_file)
                extern.run(cmd)
            with open(rev_file, 'w') as rev_aln:
                logging.debug("Writing reverse direction reads to %s" % rev_file)
                for record in reverse:
                    if record.id and record.seq:
                        rev_aln.write('>' + record.id + '\n')
                        rev_aln.write(str(record.seq.reverse_complement()) + '\n')
            self.hmmalign_sequences(self.aln_hmm, rev_file, rev_conv_file)
            conv_files = [for_conv_file, rev_conv_file]
            return conv_files
        # If there are only forward reads, just hmmalign and be done with it.
        else:
            self.hmmalign_sequences(self.aln_hmm, input_path, for_conv_file)
            conv_files = [for_conv_file]

            return conv_files

    def hmmalign_sequences(self, hmm, sequences, output_file):
        '''Run hmmalign and convert output to aligned fasta format

        Parameters
        ----------
        hmm: str
            path to hmm file
        sequences: str
            path to file of sequences to be aligned
        output_file: str
            write sequences to this file

        Returns
        -------
        nothing
        '''

        cmd = 'hmmalign --trim %s %s | seqmagick convert --input-format stockholm --output-format fasta - %s' % (hmm,
                                                                                           sequences,
                                                                                           output_file)

        extern.run(cmd)

    def makeSequenceBinary(self, sequences, fm):
        cmd = 'makehmmerdb %s %s' % (sequences, fm)
        extern.run(cmd)

    def hmmsearch(self, output_path, input_path, unpack, seq_type, threads, cutoff, orfm):
        '''
        hmmsearch - Search raw reads for hits using search_hmm list

        Parameters
        ----------
        output_path : str
            path to output domtblout table
        input_path : str
            path to input sequences to search
        unpack : UnpackRawReads
            UnpackRawReads object, returns string command that will output
            sequences to stdout when called on command line
            (use: unpack.command_line())
        seq_type : str
            variable containing a string, either 'nucleotide' or 'aminoacid'.
            Tells the pipeline whether or not to call ORFs on the sequence.
            If sequence is 'nucleotide', ORFs are called. If not, no ORFs.
        threads : Integer
            Number of threads to use. Passed to HMMsearch command.
        cutoff : str
            cutoff for HMMsearch to use, either an evalue or --cut_tc, meaning
            use the TC score cutoff specified within the HMM. Passed to 
            HMMsearch command.
        orfm : OrfM
            Object that builds the command chunk for calling ORFs on sequences
            coming through as stdin. Outputs to stdout. Calls command_line
            to construct final command line string.

        Returns
        -------
        output_table_list : array of HMMSearchResult
            Includes the name of the output domtblout table given by hmmer
        '''

        # Define the base hmmsearch command.
        logging.debug("Using %i HMMs to search" % (len(self.search_hmm)))
        output_table_list = []
        if len(self.search_hmm) > 1:
            for hmm in self.search_hmm:
                out = os.path.join(os.path.split(output_path)[0], os.path.basename(hmm).split('.')[0] + '_' + os.path.split(output_path)[1])
                output_table_list.append(out)
        elif len(self.search_hmm) == 1:
            output_table_list.append(output_path)
        else:
            raise Exception("Programming error: expected 1 or more HMMs")

        # Choose an input to this base command based off the file format found.
        if seq_type == 'nucleotide':  # If the input is nucleotide sequence
            input_cmd = orfm.command_line(input_path)
        elif seq_type == 'aminoacid':  # If the input is amino acid sequence
            input_cmd = unpack.command_line()
        else:
            raise Exception('Programming Error: error guessing input sequence type')
        
        # Run the HMMsearches
        if cutoff == "--cut_tc":
            searcher = HmmSearcher(threads, cutoff) 
        else:
            searcher = HmmSearcher(threads, '--domE %s' % cutoff) 
        searcher.hmmsearch(input_cmd, self.search_hmm, output_table_list)
        hmmtables = [HMMSearchResult.import_from_hmmsearch_table(x) for x in output_table_list]
        return hmmtables

    def merge_forev_aln(self, forward_aln_list, reverse_aln_list, outputs):
        '''
        merge_forev_aln - Merges forward and reverse alignments for a given run

        Parameters
        ----------
        aln_list : array
            List of the forward and reverse alignments for each of the runs
            given to graftM. **MUST** be the following pattern:
            [forward_run1, reverse_run1, forward_run2, reverse_run2 ...]
        outputs : array
            List of paths to output file to which the merged alignments from the
            aln_list will go into. Must be exactly half the size of the aln_list
            (i.e. one output file for every forward and reverse file provided)

        Returns
        -------
        Nothing - output files are known.
        '''

        orfm_regex = OrfM.regular_expression()
        def remove_orfm_end(records):

            new_dict = {}

            for key, record in records.iteritems():
                orfmregex = orfm_regex.match(key)
                if orfmregex:
                    new_dict[orfmregex.groups(0)[0]] = record
                else:
                    new_dict[key] = record
            return new_dict


        for idx, (forward_path, reverse_path) in enumerate(zip(forward_aln_list, reverse_aln_list)):
            output_path = outputs[idx]
            logging.info('Merging pair %s, %s' % (os.path.basename(forward_path), os.path.basename(reverse_path)))
            forward_reads = SeqIO.parse(forward_path, 'fasta')
            reverse_reads = remove_orfm_end(SeqIO.to_dict(SeqIO.parse(reverse_path, 'fasta')))

            with open(output_path, 'w') as out:
                for forward_record in forward_reads:
                    regex_match = orfm_regex.match(forward_record.id)
                    if regex_match:
                        id = regex_match.groups(0)[0]
                    else:
                        id = forward_record.id
                    forward_sequence = str(forward_record.seq)
                    try:
                        reverse_sequence = str(reverse_reads[id].seq)
                        new_seq = ''
                        if len(forward_sequence) == len(reverse_sequence):
                            for f, r in zip(forward_sequence, reverse_sequence):
                                if f == r:
                                    new_seq += f
                                elif f == '-' and r != '-':
                                    new_seq += r
                                elif r == '-' and f != '-':
                                    new_seq += f
                                elif f != '-' and r != '-':
                                    if f != r:
                                        new_seq += f
                                else:
                                    new_seq += '-'
                        else:
                            logging.error('Alignments do not match')
                            raise Exception('Merging alignments failed: Alignments do not match')
                        out.write('>%s\n' % forward_record.id)
                        out.write('%s\n' % (new_seq))
                        del reverse_reads[id]
                    except:

                        out.write('>%s\n' % forward_record.id)
                        out.write('%s\n' % (forward_sequence))
                for record_id, record in reverse_reads.iteritems():
                    out.write('>%s\n' % record_id)
                    out.write('%s\n' % (str(record.seq)))

    def nhmmer(self, output_path, unpack, threads, evalue):
        '''
        nhmmer - Search input path using nhmmer

        Parameters
        ----------
        output_path : str
            A string containing the path to the input sequences
        unpack : obj
            UnpackRawReads object, returns string command that will output
            sequences to stdout when called on command line
            (use: unpack.command_line())
        threads : str
            Number of threads to run. For compiling command line.
        evalue : str
            evalue to use. For compiling commmand line.

        Returns
        -------
        output_table_list : array
            Includes the name of the output domtblout table given by hmmer
        '''
        logging.debug("Using %i HMMs to search" % (len(self.search_hmm)))
        output_table_list = []
        if len(self.search_hmm) > 1:
            for hmm in self.search_hmm:
                out = os.path.join(os.path.split(output_path)[0], os.path.basename(hmm).split('.')[0] + '_' + os.path.split(output_path)[1])
                output_table_list.append(out)
        elif len(self.search_hmm) == 1:
            output_table_list.append(output_path)
        else:
            raise Exception("Programming error: Expected 1 or more HMMs")
        input_pipe = unpack.command_line()

        searcher = NhmmerSearcher(threads, extra_args='--incE %s -E %s' % (evalue, evalue))
        searcher.hmmsearch(input_pipe, self.search_hmm, output_table_list)

        hmmtables = [HMMSearchResult.import_from_nhmmer_table(x) for x in output_table_list]

        return hmmtables, output_table_list

    def _check_euk_contamination(self, hmm_hit_tables):
        '''
        check_euk_contamination - Check output HMM tables hits reads that hit
                                  the 18S HMM with a higher bit score.

        Parameters
        ----------
        hmm_hit_tables : array
            Array of paths to the output files produced by hmmsearch or
            nhmmer.
        run_stats : dict
            A dictionary to updatewith the number of unique 18S reads and reads
            detected by both 18S and non-18S HMMs

        Returns
        -------
        euk_reads : set
            Non-redundant set of all read names deemed to be eukaryotic
        '''
        
        euk_hit_table = HMMreader(hmm_hit_tables.pop(-1))
        other_hit_tables = [HMMreader(x) for x in hmm_hit_tables]
        reads_unique_to_eukaryotes = []
        reads_with_better_euk_hit = []

        for hit in euk_hit_table.names():
            bits = []
            for hit_table in other_hit_tables:
                if hit in hit_table.names():
                    bits.append(hit_table.bit(hit))
                else:
                    reads_unique_to_eukaryotes.append(hit)
            if bits:
                if any([x for x in bits if x > euk_hit_table.bit(hit)]):
                    continue
                else:
                    reads_with_better_euk_hit.append(hit)
            else:
                continue

        if len(reads_with_better_euk_hit) == 0:
            logging.info("No contaminating eukaryotic reads detected")
        else:
            logging.info("Found %s read(s) that may be eukaryotic" % len(reads_with_better_euk_hit + reads_unique_to_eukaryotes))

        euk_reads = set(reads_with_better_euk_hit + reads_unique_to_eukaryotes)

        return euk_reads


    def _extract_multiple_hits(self, hits, reads_path, output_path):
        '''
        splits out regions of a read that hit the HMM. For example when two of
        same gene are identified within the same contig, The regions mapping to
        the HMM will be split out and written out to a new file as a new record.

        Parameters
        ----------
        hits : dict
            A dictionary where the keys are the read names, the entry for each
            is a list of lists, each containing the range within the contig
            (or read) that mapped to the HMM. e.g.:

            {'read1': [[3, 126], [305, 413]],
             'read2': [[1, 125]],
             ...

        reads_path : str
            path to reads file containing each read or contig in FASTA format to
            be opened, and split.
        output_path : str
            path to file to which split reads will be written to in FASTA
            format.

        Returns
        -------
        Nothing, output path is known.
        '''

        complement_information = {}
        try:
            reads = SeqIO.to_dict(SeqIO.parse(reads_path, "fasta"))  # open up reads as dictionary
        except:
            logging.error("Multiple sequences found with the same ID. The input sequences are either ill formated or are interleaved. \
If you provided GraftM with an interleaved sequence file, please split them into forward and reverse reads, and provide to the the appropriate \
flags (--forward, --reverse). Otherwise, it appears that you have provided sequences with redundant IDs. GraftM doesn't know how to \
deal with these, so please remove/rename sequences with duplicate keys.")
            raise InterleavedFileError()
        with open(output_path, 'w') as out:
            for read_name, entry in hits.iteritems():  # For each contig
                ranges = entry["entry"]
                complements = entry["strand"]
                index = 1
                if len(ranges) > 1:  # if there are multiple hits in that contig
                    for r, c in zip(ranges, complements):  # for each of those hits

                        new_record = reads[read_name][r[0] - 1:r[1]]  # subset the record by the span of that hit
                        new_record.id = new_record.id + '_split_%i' % index  # give that subset record a new header
                        SeqIO.write(new_record, out, "fasta")  # and write it to output
                        index += 1  # increment the split counter
                        complement_information[new_record.id]=c

                else:  # Otherwise, just write the read back to the file
                    complement_information[read_name] = entry["strand"][0] 
                    SeqIO.write(reads[read_name], out, "fasta")
        
        return complement_information

    def _extract_from_raw_reads(self, output_path, input_reads, raw_sequences_path, input_file_format, hits):
        '''
        _extract_from_raw_reads - call fxtract to extract the hit sequences
        of the hmm/diamond search from the raw sequences file. Output into
        specified file

        Parameters
        ----------
        output_path : str
            Path of the desired output file
        input_reads : list
            list of read IDS to extract.
        raw_sequences_path : str
            Path to the raw sequences
        input_file_format : var
            Variable, either FORMAT_FASTA_GZ, FORMAT_FASTQ_GZ, FORMAT_FASTQ or
            FORMAT_FASTA, denoting the format of the input sequence
        hits : dict
            A hash with the readnames as the keys and the spans as the values

        Returns
        -------
        output_path: str
            Path to file containing extracted reads.
        
        direction_information: dict
            
            {read_1: False
             ...
             read n: True}
            
            where True = Forward direction
            and False = Reverse direction
        '''

        with tempfile.NamedTemporaryFile(prefix='_raw_extracted_reads.fa') as tmp:
            # Run fxtract to obtain reads form original sequence file
            fxtract_cmd = "fxtract -H -X -f /dev/stdin " 
            if input_file_format == FORMAT_FASTA:
                cmd = "%s %s > %s" % (fxtract_cmd, raw_sequences_path, tmp.name)
            elif input_file_format == FORMAT_FASTQ_GZ:
                cmd = "%s -z %s | awk '{print \">\" substr($0,2);getline;print;getline;getline}' > %s" % (fxtract_cmd, raw_sequences_path, tmp.name)
            elif input_file_format == FORMAT_FASTA_GZ:
                cmd = "%s -z %s > %s" % (fxtract_cmd, raw_sequences_path, tmp.name)
            elif input_file_format == FORMAT_FASTQ:
                cmd = "%s %s | awk '{print \">\" substr($0,2);getline;print;getline;getline}' > %s" % (fxtract_cmd, raw_sequences_path, tmp.name)
            else:
                raise Exception("Programming error")

            process = subprocess.Popen(["bash", "-c", cmd], 
                                       stdin=subprocess.PIPE,
                                       stdout=subprocess.PIPE)
            process.communicate('\n'.join(input_reads))
            complement_info = self._extract_multiple_hits(hits, tmp.name, output_path)  # split them into multiple reads
        
        return output_path, complement_info
        

    def alignment_correcter(self, alignment_file_list, output_file_name,
                            filter_minimum=None):
        '''
        Remove lower case insertions in alignment outputs from HMM align. Give
        a list of alignments, and an output file name, and each alignment will
        be corrected, and written to a single file, ready to be placed together
        using pplacer.

        Parameters
        ----------
        alignment_file_list : array
            List of strings, each the path to different alignments from the
            inputs provided to GraftM
        output_file_name : str
            The path and filename of the output file desired.
        filter_minimum : int
            minimum number of positions that must be aligned for each sequence
        Returns
        -------
        True or False, depending if reads were written to file

        '''

        corrected_sequences = {}
        for alignment_file in alignment_file_list:
            insert_list = []  # Define list containing inserted positions to be removed (lower case characters)
            sequence_list = list(SeqIO.parse(open(alignment_file, 'r'), 'fasta'))
            for sequence in sequence_list:  # For each sequence in the alignment
                for idx, nt in enumerate(list(sequence.seq)):  # For each nucleotide in the sequence
                    if nt.islower():  # Check for lower case character
                        insert_list.append(idx)  # Add to the insert list if it is
            insert_list = list(OrderedDict.fromkeys(sorted(insert_list, reverse=True)))  # Reverse the list and remove duplicate positions
            for sequence in sequence_list:  # For each sequence in the alignment
                new_seq = list(sequence.seq)  # Define a list of sequences to be iterable list for writing
                for position in insert_list:  # For each position in the removal list
                    del new_seq[position]  # Delete that inserted position in every sequence
                corrected_sequences['>' + sequence.id + '\n'] = (''.join(new_seq) + '\n').replace('~', '-')
        
        pre_filter_count=len(corrected_sequences)

        if filter_minimum:
            corrected_sequences={key:item for key, item in corrected_sequences.iteritems() if len(item.replace('-', '')) > filter_minimum}
        
        post_filter_count=len(corrected_sequences)
        logging.info("Filtered %i short sequences from the alignment" % \
                        (pre_filter_count-post_filter_count)
                    )
        logging.info("%i sequences remaining" % post_filter_count)
        
        if len(corrected_sequences) >= 1:
            with open(output_file_name, 'w') as output_file:  # Create an open file to write the new sequences to
                for fasta_id, fasta_seq in corrected_sequences.iteritems():
                    output_file.write(fasta_id)
                    output_file.write(fasta_seq)
            return True
        else:
            return False


    def _extract_orfs(self, input_path, orfm, hit_readnames, output_path, search_method, sequence_frame_info_list=None):
        '''
        Call ORFs on a file with nucleotide sequences and extract the proteins
        whose name is in `hit_readnames`.

        Parameters
        ----------
        input_path : str
            Path to input nucleotide sequences in FASTA format.
        orfm: obj
            graftm.OrfM object with parameters already set
        hit_readnames : str
            path to a file containin the readnames of hits to the HMM, one per
            line.
        output_path : str
            Path to output orfs into, in FASTA format.
        search_method : str
            The method for searching, either 'hmmsearch' or 'diamond'
        sequence_frame_info : list
            A dataframe (list of lists) containing readname, alignment direction
            and alignment start point information
        '''
        if search_method == "hmmsearch":
            # Build and run command to extract ORF sequences:
            orfm_cmd = orfm.command_line()
            cmd = 'fxtract -H -X -f /dev/stdin <(%s %s) > %s' % (orfm_cmd, input_path, output_path)
            process = subprocess.Popen(["bash", "-c", cmd],
                                       stdin=subprocess.PIPE,
                                       stdout=subprocess.PIPE)
            process.communicate('\n'.join(hit_readnames))

        elif search_method == "diamond":
            sequence_frame_info_dict = {x[0]:[x[1], x[2], x[3]] for x in sequence_frame_info_list}
            records = SeqIO.parse(input_path, "fasta")
            with open(output_path, 'w') as open_output_path:
                for record in records:
                    entry=sequence_frame_info_dict[record.id]
                    indfrom=(min(entry[2], entry[1])-1)
                    indto=max(entry[2], entry[1])
                    if entry[0] == False:
                        record.seq = max([x for x in record.seq[indfrom:indto].reverse_complement().translate().split("*")], key=len)
                    else:
                        record.seq = max([x for x in record.seq[indfrom:indto].translate().split("*")], key=len)
                    SeqIO.write(record, open_output_path, "fasta")
                open_output_path.flush()


    def _get_read_names(self, search_result, max_range):
        '''
        _get_read_names - loops through hmm hits and their alignment spans to
        determine if they are potentially linked (for example, if one gene in a 
        contig hits a hmm more than once, in two different conserved regions
        of that gene) and combines them into one 'hit' because they are
        technically the same. The total span of the hits deemed to be linked is
        returned.

        Parameters
        ----------
        search_result : obj
            SequenceSearchResult object with all paramaters defined. Used here
            to create rows containing information on alignment direction and
            alignment span.
        max_range : int
            Maximum range that a gene can extend within a contig. Any hits
            that extend beyond this length cannot be linked. max_range is
            set as 1.5 X the average length of all full length genes used
            in the search database. This is defined in the CONTENTS.json file
            within a gpkg.
        Returns
        -------
            Dictionary where keys are the contig/read name. The value for each
            entry is an array lists, one per hit in each contig, each with the
            span (min and max) of the alignment.
        '''

        splits = {}  # Define an output dictionary to be filled
        spans = []
        for result in search_result:  # Create a table (list of rows contain span, and complement information
            spans += list(
                         result.each(
                                    [SequenceSearchResult.QUERY_ID_FIELD,
                                        SequenceSearchResult.ALIGNMENT_DIRECTION,
                                        SequenceSearchResult.HIT_FROM_FIELD,
                                        SequenceSearchResult.HIT_TO_FIELD,
                                        SequenceSearchResult.QUERY_FROM_FIELD,
                                        SequenceSearchResult.QUERY_TO_FIELD]
                                       )
                         )

        for hit in spans:  # For each of these rows (i.e. hits)
            i = hit[0]  # set id to i
            c = hit[1]  # set complement to c
            ft = [min(hit[2:4]), max(hit[2:4])]  # set span as ft (i.e. from - to) - This is the amount covering the query
            qs = [min(hit[4:6]), max(hit[4:6])]  # seq the query span to qs - This is the amount covering the HMM

            if ft[0] == ft[1]: continue  # if the span covers none of the query, skip that entry (seen this before)

            if i not in splits:  # If the hit hasnt been seen yet
                splits[i] = {'span'       : [ft],
                             'strand'     : [c],
                             'query_span' : [qs]}  # add the span and complement as new entry

            else:  # otherwise (if it has been seen)
                for idx, entry in enumerate(splits[i]['span']):  # for each previously added entry
                    if splits[i]['strand'][idx] != c:  # If the hit is on the same complement strand
                        splits[i]['span'].append(ft)  # Add the new range to be split out in the future
                        splits[i]['strand'].append(c)  # Add the complement strand as well
                        splits[i]['query_span'].append(qs)
                        break

                    previous_qs      = splits[i]['query_span'][idx] # Get the query span of the previous hit

                    previous_q_range = set(range(previous_qs[0], previous_qs[1])) # Get the range of each
                    current_q_range  = set(range(qs[0], qs[1]))
                    query_overlap    = set(previous_q_range).intersection(current_q_range) # Find the intersection between the two ranges

                    previous_ft_span = set(range(entry[0], entry[1]))
                    current_ft_span  = set(range(ft[0], ft[1]))
                    if any(query_overlap): # If there is an overlap
                        ####################################################
                        # if the span over the actual read that hit the HMM
                        # for each hit overlap by > 25%, they are considered
                        # the same hit, and ignored
                        ####################################################
                        intersection_fraction = float(len(previous_ft_span.intersection(current_ft_span)))

                        if intersection_fraction / float(len(previous_ft_span)) >= PREVIOUS_SPAN_CUTOFF:
                            break
                        elif intersection_fraction / float(len(current_ft_span)) >= PREVIOUS_SPAN_CUTOFF:
                            break
                        else: # else (i.e. if the hit covers less that 25% of the sequence of the previous hit)
                            ####################################################
                            # If they made it this far, it means that the hits do not overlap.
                            # But one last check must be made to ensure they do not cover the same
                            # region in the HMM.
                            ####################################################
                            if len(query_overlap) > (len(current_q_range)*PREVIOUS_SPAN_CUTOFF): # if the overlap on the query HMM does not span over 25%
                                if (idx+1) == len(splits[i]['span']):
                                    splits[i]['span'].append(ft) # Add from-to as another entry, this is another hit.
                                    splits[i]['strand'].append(c) # Add strand info as well
                                    splits[i]['query_span'].append(qs)
                                    break # And break

                    if min(entry) < min(ft):  # if/else to determine which entry comes first (e.g. 1-5, 6-10 not 6-10, 1-5)
                        if max(ft) - min(entry) < max_range:  # Check if they lie within range of eachother
                            entry[1] = max(ft)  # ammend the entry if they are
                            break  # And break the loop
                    else:
                        if max(entry) - min(ft) < max_range:  # Check if they lie within range of eachother
                            entry[0] = min(ft)  # ammend the entry if they are
                            break  # And break the loop

                else:  # if no break occured (no overlap)
                    splits[i]['span'].append(ft)  # Add the new range to be split out in the future
                    splits[i]['strand'].append(c)  # Add the complement strand as well
                    splits[i]['query_span'].append(qs)
        return {key: {"entry":entry['span'], 'strand': entry['strand']} for key, entry in splits.iteritems()}  # return the dict, without strand information which isn't required.

    def _check_for_slash_endings(self, readnames):
        '''
        Provide a list of read names to be checked for the /1 or /2 endings
        which cause troubles downstream when working with paired reads. This
        function checks the list for ANY read ending in /1 or /2. The reads are
        first stripped of any comment lines, using the 'split' function

        Parameters
        ----------
        readnames : list
            list of strings, each a read name to be checked for the /1 or /2
            ending

        Returns
        -------
        Boolean. True if /1 or /2 endings are found, False if not.

        '''

        # Strip comment lines, if any
        readnames = [x.split()[0] for x in readnames]

        # Check for /1 and /2 endings:
        if any(x for x in readnames if x.endswith('/1') or x.endswith('/2')):
            result = True
        else:
            result = False

        return result

    @T.timeit
    def aa_db_search(self, files, base, unpack, search_method,
                     maximum_range, threads, evalue, min_orf_length,
                     restrict_read_length, diamond_database):
        '''
        Amino acid database search pipeline - pipeline where reads are searched
        as amino acids, and hits are identified using hmmsearch or diamond
        searches

        Parameters
        ----------
        files : obj
            graftm_output_paths object.
        base : str
            The name of the input file, stripped of all suffixes, and paths.
            Used for creating file names with 'files' object.
        unpack : obj
            UnpackRawReads object, returns string command that will output
            sequences to stdout when called on command line
            (use: unpack.command_line())
        search_method : str
            The method for searching, either 'hmmsearch' or 'diamond'
        maximum_range : int
            Maximum range that a gene can extend within a contig. Any hits
            that extend beyond this length cannot be linked. max_range is defined
            as 1.5 X the average length of all full length genes used in the
            search database. This is defined in the CONTENTS.json file within a
            gpkg.
        threads : int
            Number of threads for hmmer to use
        evalue : str
            evalue cutoff for hmmer to use
        min_orf_length : int
            minimum orf length for orfm to use
        restrict_read_length : int
            orf length to retrict orfm to.
        diamond_database : str
            Path to diamond database to use when searching. Set to 'None' if not
            using diamond pipeline
        Returns
        -------
        String path to amino acid fasta file of reads that hit
        '''
        # Define outputs
        if search_method == 'hmmsearch':
            output_search_file = files.hmmsearch_output_path(base)
        elif search_method == 'diamond':
            output_search_file = files.diamond_search_output_basename(base)
        hit_reads_fasta = files.fa_output_path(base)
        hit_reads_orfs_fasta = files.orf_fasta_output_path(base)

        return self.search_and_extract_orfs_matching_protein_database(\
                                                    unpack,
                                                    search_method,
                                                    maximum_range,
                                                    threads,
                                                    evalue,
                                                    min_orf_length,
                                                    restrict_read_length,
                                                    diamond_database,
                                                    output_search_file,
                                                    hit_reads_fasta,
                                                    hit_reads_orfs_fasta)

    def search_and_extract_orfs_matching_protein_database(self,
                                                      unpack,
                                                      search_method,
                                                      maximum_range,
                                                      threads,
                                                      evalue,
                                                      min_orf_length,
                                                      restrict_read_length,
                                                      diamond_database,
                                                      output_search_file,
                                                      hit_reads_fasta,
                                                      hit_reads_orfs_fasta):
        '''As per aa_db_search() except slightly lower level. Search an
        input read set (unpack) and then extract the proteins that hit together
        with their containing nucleotide sequences.

        Parameters
        ----------
        output_search_file: str
            path to hmmsearch output table or diamond basename
        hit_reads_fasta: str
            path to nucleotide sequences containing hit proteins
        hit_reads_orfs_fasta: str
            path to hit proteins, unaligned
            
        Returns
        -------
        direction_information: dict
                    
            {read_1: False
             ...
             read n: True}
            
            where True = Forward direction
            and False = Reverse direction
        
        result: DBSearchResult object containing file locations and hit 
        information
        
        '''

        # Define method of opening sequence files to stdout
        if unpack.is_zcattable():
            clazz = ZcatOrfM
        else:
            clazz = OrfM
        orfm = clazz(min_orf_length=min_orf_length,
                     restrict_read_length=restrict_read_length)
        extracting_orfm = OrfM(min_orf_length=min_orf_length,
                      restrict_read_length=restrict_read_length)

        if search_method == 'hmmsearch':
            # run hmmsearch
            search_result = self.hmmsearch(
                                           output_search_file,
                                           unpack.read_file,
                                           unpack,
                                           unpack.sequence_type(),
                                           threads,
                                           evalue,
                                           orfm
                                           )

        elif search_method == 'diamond':
            # run diamond
            search_result = Diamond(
                                     database=diamond_database,
                                     threads=threads,
                                     evalue=evalue,
                                     ).run(
                                           unpack.read_file,
                                           unpack.sequence_type(),
                                           daa_file_basename=output_search_file
                                           )
            search_result = [search_result]

        else:  # if the search_method isn't recognised
            raise Exception("Programming error: unexpected search_method %s" % search_method)

        orfm_regex = OrfM.regular_expression()
        
        if maximum_range:
            hits = self._get_read_names(
                                        search_result,  # define the span of hits
                                        maximum_range
                                        )
        else:   
            hits = self._get_sequence_directions(search_result)
    
        orf_hit_readnames = hits.keys() # Orf read hit names
        if unpack.sequence_type() == 'nucleotide':
            hits={(orfm_regex.match(key).groups(0)[0] if orfm_regex.match(key) else key): item for key, item in hits.iteritems()}
            hit_readnames = hits.keys() # Store raw read hit names 
        else:
            hit_readnames=orf_hit_readnames
        
        hit_reads_fasta, direction_information = self._extract_from_raw_reads(
                                                       hit_reads_fasta,
                                                       hit_readnames,
                                                       unpack.read_file,
                                                       unpack.format(),
                                                       hits
                                                       )
        

        if not hit_readnames:
            hit_read_counts = [0, len(hit_readnames)]
            result = DBSearchResult(None,
                                    search_result,
                                    hit_read_counts,
                                    None)

            return result, direction_information


        if unpack.sequence_type() == 'nucleotide':
            # Extract the orfs of these reads that hit the original search

            self._extract_orfs(
                               hit_reads_fasta,
                               extracting_orfm,
                               orf_hit_readnames,
                               hit_reads_orfs_fasta,
                               search_method,
                               list(search_result[0].each([SequenceSearchResult.QUERY_ID_FIELD,
                                                           SequenceSearchResult.ALIGNMENT_DIRECTION,
                                                           SequenceSearchResult.QUERY_FROM_FIELD,
                                                           SequenceSearchResult.QUERY_TO_FIELD])
                                    )
                               )

            hit_reads_fasta = hit_reads_orfs_fasta
        slash_endings=self._check_for_slash_endings(hit_readnames)
        result = DBSearchResult(hit_reads_fasta,
                                search_result,
                                [0, len([itertools.chain(*hits.values())])],  # array of hits [euk hits, true hits]. Euk hits alway 0 unless searching from 16S
                                slash_endings)  # Any reads that end in /1 or /2     

        if maximum_range:
            n_hits = sum([len(x["strand"]) for x in hits.values()])
        else:
            n_hits = len(hits.keys())
        logging.info("%s read(s) detected" % n_hits)
                
        return result, direction_information
    

    @T.timeit
    def nt_db_search(self, files, base, unpack, euk_check,
                     search_method, maximum_range, threads, evalue):
        '''
        Nucleotide database search pipeline - pipeline where reads are searched
        as nucleotides, and hits are identified using nhmmer searches

        Parameters
        ----------
        files : obj
            graftm_output_paths object.
        base : str
            The name of the input file, stripped of all suffixes, and paths.
            Used for creating file names with 'files' object.
        unpack : obj
            UnpackRawReads object, returns string command that will output
            sequences to stdout when called on command line
            (use: unpack.command_line())
        euk_check : bool
            True indicates the sample will be checked for eukaryotic reads,
            False indicates not.
        search_method : str
            The method for searching e.g. 'hmmsearch' or 'diamond'
        maximum_range : int
            Maximum range that a gene can extend within a contig. Any hits
            that extend beyond this length cannot be linked. max_range is defined
            as 1.5 X the average length of all full length genes used in the
            search database. This is defined in the CONTENTS.json file within a
            gpkg.
        threads : str
            Number of threads for hmmer to use
        evalue : str
            Evalue cutoff for hmmer to use
        Returns
        -------
        String path to amino acid fasta file of reads that hit
        '''

        # Define outputs
        hmmsearch_output_table = files.hmmsearch_output_path(base)
        hit_reads_fasta = files.fa_output_path(base)
        return \
            self.search_and_extract_nucleotides_matching_nucleotide_database(\
                                                      unpack,
                                                      euk_check,
                                                      search_method,
                                                      maximum_range,
                                                      threads,
                                                      evalue,
                                                      hmmsearch_output_table,
                                                      hit_reads_fasta)

    def search_and_extract_nucleotides_matching_nucleotide_database(self,
                                                      unpack,
                                                      euk_check,
                                                      search_method,
                                                      maximum_range,
                                                      threads,
                                                      evalue,
                                                      hmmsearch_output_table,
                                                      hit_reads_fasta):
        '''As per nt_db_search() except slightly lower level. Search an
        input read set (unpack) and then extract the sequences that hit.

        Parameters
        ----------
        hmmsearch_output_table: str
            path to hmmsearch output table
        hit_reads_fasta: str
            path to hit nucleotide sequences
        
        Returns
        -------
        direction_information: dict
                    
            {read_1: False
             ...
             read n: True}
            
            where True = Forward direction
            and False = Reverse direction
        
        result: DBSearchResult object containing file locations and hit 
        information
        '''

        if search_method == "hmmsearch":
            # First search the reads using the HMM
            search_result, table_list = self.nhmmer(
                                                    hmmsearch_output_table,
                                                    unpack,
                                                    threads,
                                                    evalue
                                                    )


        elif search_method == 'diamond':
            raise Exception("Diamond searches not supported for nucelotide databases yet")

        
        if maximum_range:  
            
            hits = self._get_read_names(
                                        search_result,  # define the span of hits
                                        maximum_range
                                        )
        else:   
            hits = self._get_sequence_directions(search_result)

        hit_readnames = hits.keys()
        
        if euk_check:
            euk_reads = self._check_euk_contamination(table_list)
            hit_readnames = set([read for read in hit_readnames if read not in euk_reads])
            hits = {key:item for key, item in  hits.iteritems() if key in hit_readnames}
            hit_read_count = [len(euk_reads), len(hit_readnames)]
        else:
            hit_read_count = [0, len(hit_readnames)]
        
        hit_reads_fasta, direction_information = self._extract_from_raw_reads(
                                                       hit_reads_fasta,
                                                       hit_readnames,
                                                       unpack.read_file,
                                                       unpack.format(),
                                                       hits
                                                       )
        
        if not hit_readnames:
            result = DBSearchResult(None,
                                  search_result,
                                  hit_read_count,
                                  None)
        else:
            slash_endings=self._check_for_slash_endings(hit_readnames)
            result = DBSearchResult(hit_reads_fasta,
                                    search_result,
                                    hit_read_count,
                                    slash_endings)

        if maximum_range:
            n_hits = sum([len(x["strand"]) for x in hits.values()])
        else:
            n_hits = len(hits)
        logging.info("%s read(s) detected" % n_hits)
        
        return result, direction_information
        
    @T.timeit
    def align(self, input_path, output_path, directions, pipeline, 
              filter_minimum):
        '''align - Takes input path to fasta of unaligned reads, aligns them to
        a HMM, and returns the aligned reads in the output path

        Parameters
        ----------
        input_path : str
        output_path : str
        reverse_direction : dict
            A dictionary of read names, with the entries being the complement
            strand of the read (True = forward, False = reverse)
        pipeline : str
            Either "P" or "D" corresponding to the protein and nucleotide (DNA)
            pipelines, respectively.


        Returns
        -------
        N/A - output alignment path known.
        '''

        # HMMalign the forward reads, and reverse complement reads.
        alignments = self._hmmalign(input_path,
                                    directions,
                                    pipeline)
        alignment_result = self.alignment_correcter(alignments,
                                                    output_path,
                                                    filter_minimum)
        return alignment_result
        







import extern
import logging

class PrerequisiteChecker:
    package_urls = {'taxit': 'https://github.com/fhcrc/taxtastic',
             'FastTree': 'http://www.microbesonline.org/fasttree/',
             'seqmagick': 'https://github.com/fhcrc/seqmagick',
             'orfm': 'https://github.com/wwood/OrfM',
            'fxtract': 'https://github.com/ctSkennerton/fxtract',
            'pplacer': 'http://matsen.fhcrc.org/pplacer/',
            'krona': 'http://sourceforge.net/p/krona/home/krona/',
            'mafft': 'http://mafft.cbrc.jp/alignment/software/',
            'diamond': 'https://github.com/bbuchfink/diamond',
            'hmmer': 'http://hmmer.janelia.org/'}
    
    programs_to_packages = {'hmmalign': 'hmmer',
                            'hmmsearch': 'hmmer',
                            'nhmmer': 'hmmer',
                            'ktImportText': 'krona',
                            'FastTreeMP': 'FastTree'}
    
    @staticmethod
    def check_prerequisites(program_list):
        '''Given a list of executable names, check that they are available
        on the PATH, raising an exception otherwise
        
        Parameters
        ----------
        program_list: iterable of str
            list of program names to check
        '''
        uninstalled_programs = []
        for program in program_list:
            if not extern.which(program):
                uninstalled_programs.append(program)
        if uninstalled_programs:
            msg = "The following programs appear to be missing, and need to be installed before GraftM can continue:"
            logging.error(msg)
            for program in sorted(uninstalled_programs):
                try:
                    package_name = PrerequisiteChecker.programs_to_packages[program]
                    print_name = '/'.join((program, package_name))
                except KeyError:
                    package_name = program
                    print_name = program
                    
                logging.error('\t%25s\t%s' % (print_name, PrerequisiteChecker.package_urls[package_name]))
            exit(1)



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import os.path
import sys
from StringIO import StringIO
from skbio.tree import TreeNode
from string import split as _

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.tree_cleaner import TreeCleaner

class Tests(unittest.TestCase):
    
    def match(self, tree, names):
        tc  = TreeCleaner()
        return tc.match_alignment_and_tree_sequence_ids(\
                                    names, TreeNode.read(StringIO(tree)))

    def test_match_alignment_and_tree_sequence_ids(self):
        self.match('(a,(b,c));',_('a b c'))
        
    def test_match_alignment_and_tree_sequence_ids_tree_not_align(self):
        self.assertRaises(Exception, self.match, '(a,(b,c));',_('a b'))
        
    def test_match_alignment_and_tree_sequence_ids_align_not_tree(self):
        self.assertRaises(Exception, self.match, '(a,(b,c));',_('a b c d'))
        
    def test_match_alignment_and_tree_sequence_ids_underscores(self):
        self.match('(\'a_2\',(b,c));',_('a_2 b c'))
        
    def test_remove_sequences_easy(self):
        tc  = TreeCleaner()
        tree = TreeNode.read(StringIO('(((a,b)F,c)d);'))
        tc.remove_sequences(tree, ['b'])
        self.assertEqual('((c,a)d);\n', str(tree))
        
    def test_remove_sequences_underscores(self):
        tc  = TreeCleaner()
        tree = TreeNode.read(StringIO("(((a,b),(c_yeh,L))d);"))
        tc.remove_sequences(tree, ['c_yeh'])
        self.assertEqual('(((a,b),L)d);\n', str(tree))
        
        
    def test_remove_sequences_with_named_internal_nodes(self):
        tc  = TreeCleaner()
        tree = TreeNode.read(StringIO("('Asulf_Archaeoglobus.1_2280~2522125074':7.17,(('Afulgi_764~2528311132':0.0,'CP006577_764~2588253768':0.0):0.0,'AE000782_746~638154502':0.0)'s__Archaeoglobus fulgidus':7.555):1.461;\n"))
        tc.remove_sequences(tree, 
                            ['CP006577_764~2588253768',
                             'Afulgi_764~2528311132'])
        self.assertEqual("('Asulf_Archaeoglobus.1_2280~2522125074':7.17,'AE000782_746~638154502':7.555):1.461;\n", str(tree))

if __name__ == "__main__":
    unittest.main()




# System imports
import logging
from skbio import TreeNode
from graftm.reannotator import Reannotator
from graftm.tree_decorator import TreeDecorator

class Decorator:
    '''re-root a tree and decorate it using old taxonomy, for the graftM 
    decorate pipeline.
    
    ALL trees provided to this class must be in newick format. No other formats 
    are supported. If any taxonomic decoration is already in the tree to be 
    decorated by this class is WILL be overwritten. Of course, bootstrap values 
    will remain untouched.'''

    def __init__(self, **kwargs):
        '''
        Parameters
        ----------
        reference_tree_path: str
            Path to the file containing the reference tree, which is used to
            retroot the tree tree provided to tree
        old_tree_path: str
            Path to the file containing the tree to be re-rooted. This tree will
            be rerooted at the same position as the tree porovided to the 
            reference_tree
        '''
        reference_tree_path = kwargs.pop('reference_tree_path', None)
        tree_path = kwargs.pop('tree_path')
        
        logging.debug("Importing old tree from file: %s" 
                        % tree_path)
        self.tree = TreeNode.read(open(tree_path))  
        
        if reference_tree_path:
            logging.debug("Importing reference tree from file: %s" 
                            % reference_tree_path)
            self.reference_tree = TreeNode.read(open(reference_tree_path, "r"))   
        else:
            self.reference_tree = reference_tree_path
        if len(kwargs) > 0:
            raise Exception("Unexpected arguments provided to Decorator class: %s" % kwargs)
        
    def _reroot(self):
        '''Run the re-rooting algorithm in the Reannotator class.'''
        reannotator = Reannotator()
        self.tree = reannotator._reroot_tree_by_old_root(self.reference_tree, 
                                                         self.tree)
        
    def main(self, taxonomy, output_tree, output_tax, no_unique_tax,
             decorate, seqinfo):
        '''Decorate and if necessary, re-root the tree. If an old reference tree
        is provided it is assumed that re-rooting is desired
        
        Parameters
        ----------
        taxonomy: str
            Path to a GreenGenes formatted, or taxtastic formatted taxonomy 
            file, containing the taxonomy of all or some if the sequences used 
            to construct the tree. This taxonomy will be used by the 
            tree_decorator.TreeDecorator class to decorate self.tree
        output_tree: str
            Path to file to which the decorated tree will be written to.
        output_tax: str
            Path to file to which the decorated taxonomy will be written to.
        seqinfo: str
            path to taxtastic seqinfo file to be used with taxonomy to annotate
            the tree.'''        
        # Reroot
        if self.reference_tree:
            self._reroot()
        # Decorate
        td =  TreeDecorator(self.tree,
                            taxonomy,
                            seqinfo)
        if decorate:
            td.decorate(output_tree, output_tax, no_unique_tax)
        else:
            logging.info("Writing tree to file: %s" % (output_tree))
            self.tree.write(
                            output_tree,
                            format = "newick"   
                            )


import logging

class DBSearchResult:
    '''
    DBSearchResult - Class for containing results from search pipeline in GraftM
    '''
    def __init__(self, output_reads, search_result, hit_read_count, slash_endings):
        self.output_reads   = output_reads
        self.search_result = search_result
        self.hit_count     = hit_read_count
        self.slash_endings = slash_endings
    
    def hit_fasta(self): # Return the path to the fasta file of hits
        return self.output_reads
    
    def search_objects(self): # Return SequenceSearchResult object with parameters defined
        return self.search_result
    
    def hit_count(self): # Return the number of hits found
        return self.hit_count  
    
    def slash_endings(self): # Return True if sequence headers end in /1 or /2, False if not
        return self.slash_endings
    



import extern
import os
import json
import logging
import re

from Bio import SeqIO

from graftm.timeit import Timer
from graftm.classify import Classify
from graftm.housekeeping import HouseKeeping
T=Timer()


class Pplacer:
    ### Contains function related to processing alignment files to jplace files
    ### and running comparisons between forward and revere reads if reverse
    ### reads are provided.

    def __init__(self, refpkg):
        self.refpkg = refpkg
        self.hk = HouseKeeping()
        
    # Run pplacer
    def pplacer(self, output_file, output_path, input_path, threads):
        ## Runs pplacer on concatenated alignment file
        cmd = "pplacer -j %s --verbosity 0 --out-dir %s -c %s %s" % (str(threads), output_path, self.refpkg, input_path) # Set command
        extern.run(cmd)
        output_path = '.'.join(input_path.split('.')[:-1]) + '.jplace'
        return output_path

    def alignment_merger(self, alignment_files, output_alignment_path):
        ## Concatenate aligned read_files into one file. Each read with it's
        ## own unique identifier assigning it to a particular origin file
        alias_hash = {} # Set up a hash with file names and their unique identifier
        file_number = 0 # file counter (unique identifier)
        with open(output_alignment_path, 'w') as output:
            for alignment_file in alignment_files: # For each alignment
                alignments = list(SeqIO.parse(open(alignment_file, 'r'), 'fasta')) # read list
                for record in alignments: # For each record in the read list
                    record.id = record.id + '_' + str(file_number) # append the unique identifier to the record id
                SeqIO.write(alignments, output, "fasta") # And write the reads to the file
                alias_hash[str(file_number)] = {'output_path': os.path.join(os.path.dirname(alignment_file),'placements.jplace'),
                                                'place': []}
                file_number += 1
        return alias_hash

    def jplace_split(self, jplace_file, alias_hash):
        ## Split the jplace file into their respective directories

        # Load the placement file
        placement_file = json.load(open(jplace_file))

        # Parse the placements based on unique identifies appended to the end
        # of each read
        for placement in placement_file['placements']: # for each placement
            hash = {} # create an empty hash
            for alias in alias_hash: # For each alias, append to the 'place' list each read that identifier
                hash = {'p': placement['p'],
                        'nm': [nm for nm in placement['nm'] if nm[0].split('_')[-1] == alias]}
                alias_hash[alias]['place'].append(hash)

        # Write the jplace file to their respective file paths.
        jplace_path_list = []
        for alias in alias_hash:
            output = {'fields'     : placement_file['fields'],
                      'version'    : placement_file['version'],
                      'tree'       : placement_file['tree'],
                      'placements' : alias_hash[alias]['place'],
                      'metadata'   : placement_file['metadata']}
            with open(alias_hash[alias]['output_path'], 'w') as output_path:
                json.dump(output, output_path, ensure_ascii=False)
            jplace_path_list.append(alias_hash[alias]['output_path'])
        return jplace_path_list
    
    @T.timeit
    def place(self, reverse_pipe, seqs_list, resolve_placements, files, args,
              slash_endings, tax_descr):
        '''
        placement - This is the placement pipeline in GraftM, in aligned reads 
                    are placed into phylogenetic trees, and the results interpreted.
                    If reverse reads are used, this is where the comparisons are made
                    between placements, for the summary tables to be build in the
                    next stage.
         
        Parameters
        ----------
        reverse_pipe : bool
            True: reverse reads are placed separately
            False: no reverse reads to place.
        seqs_list : list
            list of paths to alignment fastas to be placed into the tree
        resolve_placements : bool
            True:resolve placements to their most trusted taxonomy
            False: classify reads to their most trusted taxonomy, until the 
                   confidence cutoff is reached. 
        files : list
            graftM output file name object
        args : obj
            argparse object
        Returns
        ------- 
        trusted_placements : dict
            dictionary of reads and their trusted placements
        '''
        trusted_placements = {}
        
        # Merge the alignments so they can all be placed at once.
        alias_hash = self.alignment_merger(seqs_list, files.comb_aln_fa())
        
        # Run pplacer on merged file
        jplace = self.pplacer(files.jplace_output_path(), args.output_directory, files.comb_aln_fa(), args.threads)
        logging.info("Placements finished")

        # Split the jplace file
        self.jplace_split(jplace, alias_hash)
        
        #Read the json of refpkg
        logging.info("Reading classifications")
        classifications=Classify(tax_descr).assignPlacement(
                                                           jplace, 
                                                           args.placements_cutoff, 
                                                           'reads', 
                                                           resolve_placements
                                                           )
        self.hk.delete([jplace])# Remove combined split, not really useful
        logging.info("Reads classified")
        # If the reverse pipe has been specified, run the comparisons between the two pipelines. If not then just return.
        
        for idx, file in enumerate(seqs_list):
            
            if reverse_pipe:
                base_file=os.path.basename(file).replace('_forward_hits.aln.fa', '')
                forward_gup=classifications.pop(sorted(classifications.keys())[0]) 
                reverse_gup=classifications.pop(sorted(classifications.keys())[0])
                seqs_list.pop(idx+1)
                placements_hash = Compare().compare_placements(
                                                               forward_gup,
                                                               reverse_gup,
                                                               args.placements_cutoff,
                                                               slash_endings,
                                                               base_file
                                                               )
                trusted_placements[base_file]=placements_hash['trusted_placements']
               
            else: # Set the trusted placements as
                base_file=os.path.basename(file).replace('_hits.aln.fa', '')
                trusted_placements[base_file]={}
                for read, entry in classifications[str(idx)].iteritems():
                    trusted_placements[base_file][read] = entry['placement']
        
        return trusted_placements

class Compare:
    ### Functions for comparing forward and reverse read hits and placements

    def __init__(self): pass

    def _compare_hits(self, forward_reads, reverse_reads, file_name, slash_endings):
        ## Take a paired read run, and compare hits between the two, report the
        ## number of hits each, the crossover, and a
        
        def remove_endings(read_list, slash_endings):
            orfm_regex = re.compile('^(\S+)_(\d+)_(\d)_(\d+)')
            d = {}
            for read in read_list:
                orfm_match=orfm_regex.match(read)
                
                if orfm_match:
                    if slash_endings:
                        new_read=orfm_match.groups(0)[0][:-2]
                    else:
                        new_read=orfm_match.groups(0)[0]
                elif slash_endings:
                    new_read=read[:-2]
                else:
                    new_read=read
                d[new_read]=read
            return d
        
        forward_reads=remove_endings(forward_reads, slash_endings)
        reverse_reads=remove_endings(reverse_reads, slash_endings)
        # Report and record the crossover
        crossover_hits = [x for x in forward_reads.keys() if x in reverse_reads.keys()]

        # Check if there are reads to continue with.
        if len(crossover_hits) > 0:
            logging.info("%s reads found that crossover in %s" % (str(len(crossover_hits)),
                                                                       file_name))
        elif len(crossover_hits) == 0:
            logging.info("%s reads found that crossover in %s, No reads to use!" % (str(len(crossover_hits)),
                                                                                           file_name))            
        else:
            raise Exception
        # Return the hash

        return crossover_hits, forward_reads, reverse_reads

    def compare_placements(self, forward_gup, reverse_gup, placement_cutoff, slash_endings, base_file):
        ## Take guppy placement file for the forward and reverse reads, compare
        ## the placement, and make a call as to which is to be trusted. Return
        ## a list of trusted reads for use by the summary step in GraftM
        # Report and record trusted placements        
        crossover, for_dict, rev_dict = self._compare_hits(forward_gup.keys(), 
                                                           reverse_gup.keys(), 
                                                           base_file, 
                                                           slash_endings)
        
        comparison_hash = {'trusted_placements': {}} # Set up a hash that will record info on each placement
        for read in crossover:
            f_read = for_dict[read]
            r_read = rev_dict[read]
            # Check read was placed
            if forward_gup.get(f_read) is None or reverse_gup.get(r_read) is None:
                logging.info('Warning: %s was not inserted into tree' % str(f_read))
                continue # Skip read for now
            comparison_hash[read] = {} # make an entry for each read
            comparison_hash['trusted_placements'][read] = [] # Set up a taxonomy entry in trusted placements

            if len(forward_gup[f_read]['placement']) == len(reverse_gup[r_read]['placement']): # If the level of placement matches
                comparison_hash[read]['rank_length_match'] = True # Store True
            elif len(forward_gup[f_read]['placement']) != len(reverse_gup[r_read]['placement']):
                comparison_hash[read]['rank_length_match'] = False # Otherwise store False
            else:
                raise Exception('Programming Error: Comparison of placement resolution')
            for idx, (f_rank, r_rank) in enumerate(zip(forward_gup[f_read]['placement'], reverse_gup[r_read]['placement'])): # For the each rank in the read placement                    
                if f_rank == r_rank: # If the classification at this rank matches
                    comparison_hash[read]['all_ranks_match'] = True # Maintain the all ranks match are true
                    if comparison_hash[read]['rank_length_match']: # If both reads are to the same resolution
                        comparison_hash['trusted_placements'][read].append(f_rank) # Just append that rank
                    elif not comparison_hash[read]['rank_length_match']: # But if they are not at the same resolution
                        # check if we've reached the end of the forward or reverse taxonomy list
                        # and if we have, append the tail end of one with the longer taxonomy, because
                        # we can trust these placements are past the overall threshold,
                        # but one has higher resolution and so is more useful to us.
                        if len(forward_gup[f_read]['placement'][idx:]) == 1:
                            comparison_hash['trusted_placements'][read] += reverse_gup[r_read]['placement'][idx:]
                        elif len(reverse_gup[r_read]['placement'][idx:]) == 1:
                            comparison_hash['trusted_placements'][read] += forward_gup[f_read]['placement'][idx:]
                        elif len(forward_gup[f_read]['placement'][idx:]) > 1 and len(reverse_gup[r_read]['placement'][idx:]) > 1:
                            comparison_hash['trusted_placements'][read].append(f_rank)
                        else:
                            raise Exception('Programming Error')
                elif f_rank != r_rank: # Otherwise if the classification doesn't match
                    comparison_hash[read]['all_ranks_match'] = False # Store that all ranks do not match up
                    forward_confidence = forward_gup[f_read]['confidence'][idx] # Store confidence values for both directions
                    reverse_confidence = reverse_gup[r_read]['confidence'][idx]
                    if float(forward_confidence) > float(reverse_confidence): # If the forward read has more confidence
                        comparison_hash['trusted_placements'][read] += forward_gup[f_read]['placement'][idx:] # Store the taxonomy of that read from that point on
                        break
                    elif float(reverse_confidence) > float(forward_confidence): # Do the opposite if reverse read has more confidence
                        comparison_hash['trusted_placements'][read] += reverse_gup[r_read]['placement'][idx:]
                        break
                    elif float(reverse_confidence) == float(forward_confidence): # If they are of the same value
                        break # Do nothing, because a decision cannot be made if the confidence is equal.
                    else:
                        raise Exception('Programming Error: Comparing confidence values')
                else:
                    raise Exception('Programming Error: Comparison of placement resolution')               
             
        return comparison_hash # Return the hash




class MalformedGreenGenesTaxonomyException(Exception):
    pass

class GreenGenesTaxonomy:
    # A hash of name to array of taxonomy strings
    taxonomy = {}
    
    def __init__(self, taxonomy={}):
        self.taxonomy = taxonomy
        
    @staticmethod
    def read(input_taxonomy_io):
        '''Parse in a taxonomy from the given I/O.
        
        Parameters
        ----------
        input_taxonomy_io: io
            an open io object that is iterable. This stream is neither opened
            nor closed by this method
            
        Returns
        -------
        A GreenGenesTaxonomy instance with the taxonomy parsed in
        
        Raises
        ------
        MalformedGreenGenesTaxonomyException: if there is something amiss'''
        tax = {}
        for line in input_taxonomy_io:
            splits = line.split("\t")
            if len(splits) != 2:
                raise MalformedGreenGenesTaxonomyException("Unexpected number of tab-separated fields found in taxonomy file, on line %s" % line)
            name = splits[0]
            taxonomy = [t.strip() for t in splits[1].split(';')]
            while len(taxonomy) > 0 and taxonomy[-1] == '':
                taxonomy = taxonomy[:-1]
            for lineage in taxonomy:
                if lineage == '':
                    raise MalformedGreenGenesTaxonomyException("Encountered a taxonomy string with the middle of the taxonomy string missing: %s" % line)
            
            if name in tax:
                raise MalformedGreenGenesTaxonomyException("Duplicate definition of taxonomy for %s" % name)
            tax[name] = taxonomy
        return GreenGenesTaxonomy(tax)
    
    @staticmethod
    def read_file(input_filename):
        '''Like read() except uses a file rather than an IO stream, for convenience'''
        with open(input_filename) as f:
            g = GreenGenesTaxonomy.read(f)
        return g


import numpy as np
from biom.table import Table
import tempfile
import extern
import logging

class Stats_And_Summary:

    def __init__(self): pass

    def coverage_of_hmm(self, hmm, count_table, coverage_table, avg_read_length):

        for line in open(hmm):

            if line.startswith('LENG'):
                length = line.split()[1]

        with open(coverage_table, 'w') as ct:
            write = ['#ID     20110816_S1D    ConsensusLineage\n']
            for line in open(count_table):

                    if line.startswith('#'):
                        continue

                    splt = line.split()

                    cov = str(round((float(splt[1])*float(avg_read_length)) / float(length), 3))

                    write.append("%s\t%s\t%s\n" % (
                                    splt[0],
                                    cov,
                                    splt[2]))
            for entry in write:
                ct.write(entry)

    def readTax(self, placements, output_path):
        with open(output_path, 'w') as out:
            for read, tax in placements.iteritems():
                out.write("%s\t%s\n" % (read, '; '.join(tax)))

    def build_basic_statistics(self, times, hit_read_count_list, placed_reads, base_list, output):
        
        output_lines = ["Basic run statistics (count):"]
        output_lines.append("Files:\t%s" % '\t'.join(base_list))
        if any([x[0] for x in hit_read_count_list if x[0] > 0]):
            output_lines.append("18S reads filtered:\t%s" % '\t'.join([str(x[0]) for x in hit_read_count_list]))
        output_lines.append("reads detected:\t%s" % '\t'.join([str(x[1]) for x in hit_read_count_list]))
        output_lines.append("reads placed in tree:\t%s" % '\t'.join([str(x) for x in placed_reads]))   
        output_lines.append("Runtime (seconds):")
        output_lines.append("Search step:\t%s" % times[0])
        output_lines.append("Alignment step:\t%s" % times[1])
        output_lines.append("Tree insertion step:\t%s" % times[2])

        
        with open(output, 'w') as stats_file:
            for line in output_lines:
                stats_file.write(line + '\n')

    def _iterate_otu_table_rows(self, read_taxonomies):
        '''yield that which is required for an OTU table: taxonomy, and
        count of that taxonomy in each sample as an array
        
        Parameters
        ---------
        read_taxonomies:
            a list of hashes, where the position in the list corresponds to the 
            sample list, the key is the read name, and the value is an array
            of taxonomic info
            
        Yield
        -----
        OTU_ID: int, made up by this function
        taxonomy: array where each element is a taxonomic level
        count: number of observations of that taxonomy in each sample respectively
            
        Return
        ------
        Nothing, use this as an iterator'''
        
        taxonomy_string_to_taxonomy_array = {}
        taxonomy_string_to_counts = {}
        sample_index = 0
        num_samples = len(read_taxonomies)
        for read_to_taxonomy in read_taxonomies: # For each sample
            for taxonomy_array in read_to_taxonomy.values(): # For each read
                taxonomy_string = '; '.join(taxonomy_array)
                if taxonomy_string_to_taxonomy_array.has_key(taxonomy_string):
                    if taxonomy_string_to_taxonomy_array[taxonomy_string] != taxonomy_array:
                        raise Exception("Programming error: two different taxonomies had same taxonomy string")
                    try:
                        taxonomy_string_to_counts[taxonomy_string][sample_index] += 1
                    except KeyError:
                        taxonomy_string_to_counts[taxonomy_string][sample_index] = 1
                else:
                    taxonomy_string_to_taxonomy_array[taxonomy_string] = taxonomy_array
                    taxonomy_string_to_counts[taxonomy_string] = [0]*num_samples
                    taxonomy_string_to_counts[taxonomy_string][sample_index] = 1
            sample_index += 1
            
        otu_id = 1
        for tax_string, counts_array in taxonomy_string_to_counts.iteritems():
            array = []
            for i in range(num_samples):
                try:
                    array.append(counts_array[i])
                except IndexError:
                    array.append(0)
            yield otu_id,\
                taxonomy_string_to_taxonomy_array[tax_string],\
                array
            otu_id += 1
            
    def write_biom(self, sample_names, read_taxonomies, biom_file_io):
        '''Write the OTU info to a biom IO output stream
        
        Parameters
        ----------
        sample_names: String
            names of each sample (sample_ids for biom)
        read_taxonomies: Array of hashes as per _iterate_otu_table_rows()
        biom_file_io: io
            open writeable stream to write biom contents to
            
        Returns True if successful, else False'''
        counts = []
        observ_metadata = []
        otu_ids = []
        for otu_id, tax, count in self._iterate_otu_table_rows(read_taxonomies):
            if len(count) != len(sample_names):
                raise Exception("Programming error: mismatched sample names and counts")
            counts.append(count)
            observ_metadata.append({'taxonomy': tax})
            otu_ids.append(str(otu_id))
        table = Table(np.array(counts),
                      otu_ids, sample_names, observ_metadata,
                      [{}]*len(sample_names), table_id='GraftM Taxonomy Count Table')
        try:
            table.to_hdf5(biom_file_io, 'GraftM graft')
            return True
        except RuntimeError as e:
            logging.warn("Error writing BIOM output, file not written. The specific error was: %s" % e)
            return False

    def write_tabular_otu_table(self, sample_names, read_taxonomies, combined_output_otu_table_io):
        '''A function that takes a hash of trusted placements, and compiles them
        into an OTU-esque table.'''
        delim = u'\t'
        combined_output_otu_table_io.write(delim.join(['#ID', 
                                                       delim.join(sample_names),
                                                       'ConsensusLineage']))
        combined_output_otu_table_io.write(u"\n")
        for otu_id, tax, counts in self._iterate_otu_table_rows(read_taxonomies):
            combined_output_otu_table_io.write(delim.join(\
                (str(otu_id),
                 delim.join([str(c) for c in counts]),
                 '; '.join(tax)))+"\n")
            
    def write_krona_plot(self, sample_names, read_taxonomies, output_krona_filename):
        '''Creates krona plot at the given location. Assumes the krona executable
        ktImportText is available on the shell PATH'''
        tempfiles = []
        for n in sample_names:
            tempfiles.append(tempfile.NamedTemporaryFile(prefix='GraftMkronaInput', suffix=n))
        
        delim=u'\t'
        for _, tax, counts in self._iterate_otu_table_rows(read_taxonomies):
            for i, c in enumerate(counts):
                if c != 0:
                    tempfiles[i].write(delim.join((str(c),
                                                  delim.join(tax)
                                                  ))+"\n")
                    
        for t in tempfiles:
            t.flush()
        
        cmd = ["ktImportText",'-o',output_krona_filename]
        for i, tmp in enumerate(tempfiles):
            cmd.append(','.join([tmp.name,sample_names[i]]))

        # run the actual krona
        cmd = ' '.join(cmd)
        extern.run(cmd)

        # close tempfiles
        for t in tempfiles:
            t.close()

        
        


import time

class Timer:
    
    def __init__(self): pass

    def timeit(self, method):
        def timed(*args, **kw):
            ts = time.time()
            result = method(*args, **kw)
            te = time.time()
            return round(te-ts, 2), result
        return timed


#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import os.path
import sys
from StringIO import StringIO
import tempfile

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.greengenes_taxonomy import GreenGenesTaxonomy, MalformedGreenGenesTaxonomyException

class Tests(unittest.TestCase):
    def test_read_hello_world(self):
        self.assertEqual({'seq1': ['bacteria','cyanobacteria']},\
                          GreenGenesTaxonomy.read(StringIO('seq1\tbacteria; cyanobacteria')).taxonomy)        

    def test_read_semicolon_no_space(self):
        self.assertEqual({'seq1': ['bacteria','cyanobacteria']},\
                          GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria')).taxonomy)
        
    def test_raises_when_incorrect_num_fields(self):
        with self.assertRaises(MalformedGreenGenesTaxonomyException):
            GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria\n'\
                                           'seq2\n'
                                           ))

    def test_ok_when_taxonomy_empty(self):
        self.assertEqual({'seq1': ['bacteria','cyanobacteria'], 'seq2': []},\
            GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria\n'\
                                           'seq2\t\n'
                                           )).taxonomy)
        
    def test_raises_when_duplicate_names(self):
        with self.assertRaises(MalformedGreenGenesTaxonomyException):
            GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria\n'\
                                           'seq1\tbacteria;cyanobacteria\n'
                                           ))
            
    def test_raises_when_missing_middle(self):
        with self.assertRaises(MalformedGreenGenesTaxonomyException):
            GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria\n'\
                                           'seq2\tbacteria;;cyanobacteria\n'
                                           ))
            
    def test_removes_empties_at_end(self):
        self.assertEqual({'seq1': ['bacteria','cyanobacteria'], 'seq2': ['bacteria','bluebacteria']},\
            GreenGenesTaxonomy.read(StringIO('seq1\tbacteria;cyanobacteria;\n'\
                                           'seq2\tbacteria;bluebacteria;;\n'
                                           )).taxonomy)
            
    def test_input_file(self):
        with tempfile.NamedTemporaryFile(prefix='graftm_greengenes_tax_testing') as tf:
            tf.write('seq1\tbacteria;cyanobacteria')
            tf.flush()
            self.assertEqual({'seq1': ['bacteria','cyanobacteria']},\
                      GreenGenesTaxonomy.read_file(tf.name).taxonomy)

if __name__ == "__main__":
    unittest.main()



import skbio.tree

class TreeCleaner:
    def clean_newick_file_for_fasttree_input(self, tree):
        '''Modify the given skbio.tree.TreeNode so that it can be output and fed
        into FastTree'''
        for n in tree.non_tips(include_self=True): n.name=None
        #convert underscores to spaces so skbio converts them back
        # without using single quotes for the names. See
        # https://github.com/biocore/scikit-bio/issues/934
        for n in tree.tips(): n.name = n.name.replace('_', ' ')
            
    def match_alignment_and_tree_sequence_ids(self, sequence_names, tree):
        '''Check to make sure that the sequences specified in the alignment
        and the tree are the same, otherwise raise an Exception detailing
        the problem for the user to fix'''
        
        tip_names_count = {}
        for t in tree.tips():
            # replace spaces with underscores as skbio interprets unquoted underscores
            # as spaces, where fasttree doesn't (I think)
            name = t.name.replace(' ','_')
            if name in tip_names_count:
                raise Exception("Duplicate tip name found in tree: '%s'" % name)
            else:
                tip_names_count[name] = 1
        for name in sequence_names:
            if name not in tip_names_count:
                raise Exception("The alignment sequence '%s' was found in the alignment but not the tree" % name)
            elif tip_names_count[name] > 1:
                raise Exception("Found duplicate sequence name '%s'" % name)
            else:
                tip_names_count[name] += 1
        for name, count in tip_names_count.iteritems():
            if count < 2:
                raise Exception("Sequence '%s' was found in the tree but not the alignment" % name)
            
    def remove_sequences(self, tree, sequence_names):
        '''Remove sequences with in the given sequence_names array from the tree
        in place.
        
        Assumes the sequences are found in the tree, and that they are all unique.
        
        Potentially this could be better implemented using TreeNode.shear,
        but there are underscores to deal with (or not)
        '''
        for s in sequence_names:
            try:
                n = tree.find(s.replace('_',' '))
            except skbio.tree.MissingNodeError:
                # Just delete it already
                n = tree.find(s)
            if not n.parent.remove(n):
                raise Exception("Unexpectedly failed to remove '%s' from the\
                    tree. As a guess, are all the tip names unique?")
            tree.prune() #call prune() afer each remove as workaround for
            # https://github.com/biocore/scikit-bio/issues/970
        
        
        
    


import logging

from skbio.tree._tree import TreeNode
from graftm.rerooter import Rerooter

class TreeParaphyleticException(Exception): pass

class Reannotator:
    def _reroot_tree_by_old_root(self, old_tree, new_tree):
        '''reroot the new tree so that it matches the old tree's root, if 
        possible. If more than one rerooting is possible, root at the longest
        internal branch that is consistent with the root of the old_tree.
        
        Parameters
        ----------
        old_tree: skbio.TreeNode
            The old tree to try to match the root from
        new_tree: skbio.TreeNode
            tree to be rerooted in the same place as the old_tree.
            Must include at least one leaf from each side of the old_tree's
            root (matching by TreeNode.name), but may not have all leaves from
            the old tree, and can have extra leaves. 
            
        Returns
        -------
        The new_tree rooted by the root of the old tree
        
        Exceptions
        ----------
        TreeParaphyleticException
            If either of the old_tree's branches are not monophyletic in the
            new tree
        '''
        
        # make a list of the left and right leaf names that are in the new tree
        if len(old_tree.children) != 2: 
            logging.debug("Tree is not binary (root does not have exactly 2 children). Rerooting tree such that it is binary")
            old_tree=Rerooter().reroot(old_tree)
        new_tip_names = set([tip.name for tip in new_tree.tips(include_self=False)])
        old_left_tip_names = [tip.name for tip in old_tree.children[0].tips(include_self=False) if tip.name in new_tip_names]
        old_right_tip_names = [tip.name for tip in old_tree.children[1].tips(include_self=False) if tip.name in new_tip_names]
        
        # find the LCA of the lefts and the rights, giving up at the root.
        left_lca = new_tree.lowest_common_ancestor(old_left_tip_names)
        right_lca = new_tree.lowest_common_ancestor(old_right_tip_names)
        
        # if both didn't LCA before hitting the root, tree paraphyletic
        # take the first one where the LCA was hit, reroot here.
        # find the LCA of the other in the other half of the tree.
        # if failed before getting to the root, then tree paraphyletic
        # reroot on one side of the internal branch between the two LCAs
        if left_lca == new_tree:
            if right_lca == new_tree:
                raise TreeParaphyleticException("Tree paraphyletic case #1")
            else:
                new_tree = self._reroot_workaround(new_tree, right_lca)
                new_lca = new_tree.lowest_common_ancestor(old_left_tip_names)
        else:
            new_tree = self._reroot_workaround(new_tree, left_lca)
            new_lca = new_tree.lowest_common_ancestor(old_right_tip_names)
            
        if new_lca.parent is None:
            raise TreeParaphyleticException("Tree paraphyletic case #2")
        far_node = self._find_longest_internal_branch_node(new_tree, new_lca)
        new_tree = self._reroot_workaround(new_tree, far_node)
        
        return new_tree
        
    def _find_longest_internal_branch_node(self, root_node, node):
        '''return the node that has the longest branch length between the given
        node and the root
        
        Parameters
        ----------
        root_node: skbio.TreeNode
            root of the tree
        node: skbio.TreeNode
            a node from the tree
            
        Returns
        -------
        The node that has the largest length between the node and the root_node
        '''
        max_length = -1
        max_node = None
        while node.parent is not None:
            if node.length > max_length:
                max_node = node
                max_length = node.length
            node = node.parent
        return max_node
    
    def _reroot_workaround(self, tree, node):
        '''reroot the tree by adding a new node and removing the old root
        node, preserving distances. A new root node is added above the node
        with zero length, and the correct length to the sister node. The old
        root node is removed from the tree, again preserving distances.
        
        Parameters
        ----------
        tree: skbio.TreeNode
            tree to be rerooted
        node: skbio.TreeNode
            node that is part of the tree to be rerooted.
            
        Returns
        -------
        The input tree rerooted.
        '''
        if node == tree:
            raise Exception("Unexpected rooting: cannot reroot by current root")
        elif node in tree.children:
            # trivial case, just move the root across the branch
            sisters = [n for n in tree.children if n != node]
            if len(sisters) != 1: raise Exception("Unexpectedly found num sisters != 1")
            sister = sisters[0]
            new_root = TreeNode(children=[node, sister], parent=None)
            sister.length = sister.length + node.length
            sister.parent = new_root
            node.length = 0.0
            node.parent = new_root
            return new_root
        else:
            # regular case, new root is down the tree somewhere
            an_old_child = tree.children[0]
            OLD_CHILD_NAME = 'ochild'
            an_old_child.name = OLD_CHILD_NAME
            
            # create new root and reroot there
            new_root = TreeNode(length=0.0, children=[node], parent=node.parent)
            new_root.parent.children = [n for n in new_root.parent.children if n != node]+[new_root]
            new_root = tree.root_at(new_root)

            # remove leftover dummy root
            old_child_node = new_root.find(OLD_CHILD_NAME)
            nodes_to_remove = [n for n in [old_child_node,old_child_node.parent] if len(n.children) == 1]
            if len(nodes_to_remove) == 0:
                # sometimes trees do not have a separate root node,
                # I suppose.
                pass                
            elif len(nodes_to_remove) == 1:
                n = nodes_to_remove[0]
                child = n.children[0]
                grandchildren = child.children
                if len(grandchildren) != 2:
                    raise Exception("Unexpectedly found num grandchildren of dummy not != 2")
                for g in grandchildren:
                    g.parent = n
                n.length = n.length + child.length
                n.children = grandchildren                
            else:
                raise Exception("Unexpectedly found >1 child nodes")

            old_child_node.name = None
    
            return new_root
        



#!/usr/bin/env python

#=======================================================================
# Authors: Ben Woodcroft, Joel Boyd
#
# Unit tests.
#
# Copyright
#
# This is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License.
# If not, see <http://www.gnu.org/licenses/>.
#=======================================================================

import unittest
import sys
import os
import tempfile
from graftm.unpack_sequences import UnpackRawReads

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')]+sys.path
from graftm.diamond import Diamond

path_to_data = os.path.join(os.path.dirname(os.path.realpath(__file__)),'data')

class Tests(unittest.TestCase):
    _protein_query = '''>seq1 637699780 METHANOSARCINA BARKERI MCRA
SKFKKDMEVKFGGDITDKTAKFLRLGPEQDPRKVEMIKAGKE
IAEKRGIAFYNPMMHSGAPLGQRAITPYTISGTDIVCEPDDLHYVNNAAMQQMWDDIRRT
CIVGLDMAHETLEKRLGKEVTPETINHYLEVLNHAMPGAAVVQEMMVETHPALVDDCYVK
VFTGDDALADEIDKQFLIDINKEFSEEQAAQIKASIGKTSWQAIHIPTIVSRTTDGAQTS
RWAAMQIGMSFISAYAMCAGEAAVADLSFAAKHAALVSMGEMLPARRARGPNEPGGLSFG
HLSDIVQTSRVSEDPAKIALEVVGAGCMLYDQIWLGSYMSGGVGFTQYATAAYTDDILD
NNTYYDVDYINDKYNKDNKVKASLEVVKDIATESTLYGIETYEKFPTALEDHFG
GSQRATVLAAAAGVACSLATGNANAGLSGWYLSMYLHKEAWGRLGFFGFDLQDQCGATNV
LSYQGDEGLPDELRGPNYPNYAMNVGHQGGYAGIAQAAHSGRGDAFTVNPLLKVCFADDL
LPFNFAEPRREFGRGAIREFVPAGERSLVIPAK
>seq2 638201361 METHANOCALDOCOCCUS JANNASCHII MRTA
DVEKKLFLKALKEKFEEDPKEKYTKFYIFGGWRQSARKREFVEFAQKLIEKRGIPF
YNPDIGVPLGQRKLMTYKISGTDAFVEGDDLHFCNNAAIQQLVDDIKRTVIVGMDTAH
AVLEKRLGVEVTPETINEYMETINHALPGGAVVQEHMVEVHPGLVWDCYAKIFTGNDELA
DEIDKRFLIDINKEFPEEQAEQIKKYIGNRTYQVSRVPTIVVRCCDGGTVSRWSAMQIGM
SFITAYKLCAGEAAIADFSYAAKHADVIQMGMILPARRARGPNEPGGVPFGIFADIIQTS
RVSDDPAQVTLEVIGAAATFYDQVWLGSYMSGGVGFTQYASATYTDDILDDFVYYGMDY
VEKKYGLCGVKPSMEVVKDIATEVTLYGLEQYDEYPALLEDHFGGSQRAGVTAAAAGCS
VAFATGNSNAGINGWYLSQILHKEYHSRLGFYGYDLQDQCGAANSLSIRSDEGLLHECRG'''

    def test_blastp(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(self._protein_query)
            f.flush()
            d = Diamond(os.path.join(path_to_data,'diamond','mcra.faa.dmnd'))
            res = d.run(f.name, UnpackRawReads.PROTEIN_SEQUENCE_TYPE)
            # ben@ben:~/git/graftM.local/test$ diamond view -a /tmp/a
            # seq1    637699780    100.0    548    0    0    1    548    1    548    0.0e+00    1111.3
            # seq2    638201361    100.0    472    0    0    1    472    1    472    2.9e-283    963.0
            self.assertEqual(

                             [['seq1', '637699780', '100.0', '548', '0', 1, 548, 1, 548, '0.0e+00', '1111.3', True],
                              ['seq2', '638201361', '100.0', '472', '0', 1, 472, 1, 472, '2.9e-283', '963.0', True]],
                             list([x[:-1] for x in res.each(res.fields)])
                             )
            
    def test_basename(self):
        with tempfile.NamedTemporaryFile() as f:
            f.write(self._protein_query)
            f.flush()
            d = Diamond(os.path.join(path_to_data,'diamond','mcra.faa.dmnd'))
            base = 'mybase'
            daa="%s.daa" % base
            res = d.run(f.name, UnpackRawReads.PROTEIN_SEQUENCE_TYPE, daa_file_basename=base)
            # ben@ben:~/git/graftM.local/test$ diamond view -a /tmp/a
            # seq1    637699780    100.0    548    0    0    1    548    1    548    0.0e+00    1111.3
            # seq2    638201361    100.0    472    0    0    1    472    1    472    2.9e-283    963.0

            self.assertEqual(
                             [['seq1', '637699780', '100.0', '548', '0', 1, 548, 1, 548, '0.0e+00', '1111.3', True],
                              ['seq2', '638201361', '100.0', '472', '0', 1, 472, 1, 472, '2.9e-283', '963.0', True]],
                             list([x[:-1] for x in res.each(res.fields)])
                             )
            self.assertTrue(os.path.exists(daa))
            os.remove("%s.daa" % base)


if __name__ == "__main__":
    unittest.main()
